import unittest
from tuned.profiles.merger import Merger
from tuned.profiles.profile import Profile
from collections import OrderedDict

class MergerTestCase(unittest.TestCase):
	def test_merge_without_replace(self):
		merger = Merger()
		config1 = OrderedDict([
			("main", {"test_option" : "test_value1"}),
			("net", { "devices": "em0", "custom": "custom_value"}),
		])
		profile1 = Profile('test_profile1',config1)
		config2 = OrderedDict([
			('main', {'test_option' : 'test_value2'}),
			('net', { 'devices': 'em1' }),
		])
		profile2 = Profile("test_profile2",config2)

		merged_profile = merger.merge([profile1, profile2])

		self.assertEqual(merged_profile.options["test_option"],"test_value2")
		self.assertIn("net", merged_profile.units)
		self.assertEqual(merged_profile.units["net"].options["custom"],\
			"custom_value")
		self.assertEqual(merged_profile.units["net"].devices, "em1")

	def test_merge_with_replace(self):
		merger = Merger()
		config1 = OrderedDict([
			("main", {"test_option" : "test_value1"}),
			("net", { "devices": "em0", "custom": "option"}),
		])
		profile1 = Profile('test_profile1',config1)
		config2 = OrderedDict([
			("main", {"test_option" : "test_value2"}),
			("net", { "devices": "em1", "replace": True }),
		])
		profile2 = Profile('test_profile2',config2)
		merged_profile = merger.merge([profile1, profile2])

		self.assertEqual(merged_profile.options["test_option"],"test_value2")
		self.assertIn("net", merged_profile.units)
		self.assertNotIn("custom", merged_profile.units["net"].options)
		self.assertEqual(merged_profile.units["net"].devices, "em1")

	def test_merge_multiple_order(self):
		merger = Merger()
		config1 = OrderedDict([ ("main", {"test_option" : "test_value1"}),\
			("net", { "devices": "em0" }) ])
		profile1 = Profile('test_profile1',config1)
		config2 = OrderedDict([	("main", {"test_option" : "test_value2"}),\
			("net", { "devices": "em1" }) ])
		profile2 = Profile('test_profile2',config2)
		config3 = OrderedDict([	("main", {"test_option" : "test_value3"}),\
			("net", { "devices": "em2" }) ])
		profile3 = Profile('test_profile3',config3)
		merged_profile = merger.merge([profile1, profile2, profile3])

		self.assertEqual(merged_profile.options["test_option"],"test_value3")
		self.assertIn("net", merged_profile.units)
		self.assertEqual(merged_profile.units["net"].devices, "em2")



import unittest
import tempfile
import shutil
import os

import tuned.profiles as profiles
from tuned.profiles.exceptions import InvalidProfileException

class LoaderTestCase(unittest.TestCase):
	@classmethod
	def setUpClass(cls):
		cls._test_dir = tempfile.mkdtemp()
		cls._profiles_dir = cls._test_dir + '/test_profiles'
		cls._dummy_profile_dir = cls._profiles_dir + '/dummy'
		cls._dummy_profile_dir2 = cls._profiles_dir + '/dummy2'
		cls._dummy_profile_dir3 = cls._profiles_dir + '/dummy3'
		cls._dummy_profile_dir4 = cls._profiles_dir + '/dummy4'
		try:
			os.mkdir(cls._profiles_dir)
			os.mkdir(cls._dummy_profile_dir)
			os.mkdir(cls._dummy_profile_dir2)
			os.mkdir(cls._dummy_profile_dir3)
			os.mkdir(cls._dummy_profile_dir4)
		except OSError:
			pass

		with open(cls._dummy_profile_dir + '/tuned.conf','w') as f:
			f.write('[main]\nsummary=dummy profile\n')
			f.write('[test_unit]\ntest_option=hello\n')
			f.write('random_option=random\n')

		with open(cls._dummy_profile_dir2 + '/tuned.conf','w') as f:
			f.write(\
			'[main]\nsummary=second dummy profile\n')
			f.write('[test_unit]\ntest_option=hello world\n')
			f.write('secondary_option=whatever\n')

		with open(cls._dummy_profile_dir3 + '/tuned.conf','w') as f:
			f.write('[main]\nsummary=another profile\ninclude=dummy\n')
			f.write('[test_unit]\ntest_option=bye bye\n')
			f.write('new_option=add this\n')

		with open(cls._dummy_profile_dir4 + '/tuned.conf','w') as f:
			f.write(\
				'[main]\nsummary=dummy profile for configuration read test\n')
			f.write('file_path=${i:PROFILE_DIR}/whatever\n')
			f.write('script=random_name.sh\n')
			f.write('[test_unit]\ntest_option=hello world\n')
			f.write('devices=/dev/${variable1},/dev/${variable2}\n')
			f.write('[variables]\nvariable1=net\nvariable2=cpu')

	def setUp(self):
		locator = profiles.Locator([self._profiles_dir])
		factory = profiles.Factory()
		merger = profiles.Merger()
		self._loader = profiles.Loader(locator,factory,merger,None,\
			profiles.variables.Variables())

	def test_safe_name(self):
		self.assertFalse(self._loader.safe_name('*'))
		self.assertFalse(self._loader.safe_name('$'))
		self.assertTrue(self._loader.safe_name('Allowed_ch4rs.-'))

	def test_load_without_include(self):
		merged_profile = self._loader.load(['dummy','dummy2'])

		self.assertEqual(merged_profile.name, 'dummy dummy2')
		self.assertEqual(merged_profile.options['summary'],\
			'second dummy profile')
		self.assertEqual(merged_profile.units['test_unit'].\
			options['test_option'],'hello world')
		self.assertEqual(merged_profile.units['test_unit'].\
			options['secondary_option'],'whatever')

		with self.assertRaises(InvalidProfileException):
			self._loader.load([])

		with self.assertRaises(InvalidProfileException):
			self._loader.load(['invalid'])

	def test_load_with_include(self):
		merged_profile = self._loader.load(['dummy3'])

		self.assertEqual(merged_profile.name,'dummy3')
		self.assertEqual(merged_profile.options['summary'],'another profile')
		self.assertEqual(merged_profile.units['test_unit'].\
			options['test_option'],'bye bye')
		self.assertEqual(merged_profile.units['test_unit'].\
			options['new_option'],'add this')
		self.assertEqual(merged_profile.units['test_unit'].\
			options['random_option'],'random')

	def test_expand_profile_dir(self):
		self.assertEqual(self._loader._expand_profile_dir(\
			'/hello/world','${i:PROFILE_DIR}/file'),'/hello/world/file')

	def test_load_config_data(self):
		config = self._loader._load_config_data(\
			self._dummy_profile_dir4 + '/tuned.conf')

		self.assertEqual(config['main']['script'][0],\
			self._dummy_profile_dir4 + '/random_name.sh')

		self.assertEqual(config['main']['file_path'],\
			self._dummy_profile_dir4 + '/whatever')

		self.assertEqual(config['test_unit']['test_option'],\
			'hello world')

	def test_variables(self):
		config = self._loader.load(['dummy4'])
		self.assertEqual(config.units['test_unit'].devices,\
			'/dev/net,/dev/cpu')

	@classmethod
	def tearDownClass(cls):
		shutil.rmtree(cls._test_dir)



import unittest
import tempfile
import shutil
from tuned.profiles import variables, profile

class VariablesTestCase(unittest.TestCase):

	@classmethod
	def setUpClass(cls):
		cls.test_dir = tempfile.mkdtemp()

		with open(cls.test_dir + "/variables", 'w') as f:
			f.write("variable1=var1\n")

	def test_from_file(self):
		v = variables.Variables()
		v.add_from_file(self.test_dir + "/variables")
		self.assertEqual("This is var1", v.expand("This is ${variable1}"))

	def test_from_unit(self):
		mock_unit = {
			"include": self.test_dir + "/variables",
			"variable2": "var2"
		}
		v = variables.Variables()
		v.add_from_cfg(mock_unit)

		self.assertEqual("This is var1 and this is var2", v.expand("This is ${variable1} and this is ${variable2}"))

	@classmethod
	def tearDownClass(cls):
		shutil.rmtree(cls.test_dir)



__all__ = ["command_set", "command_get", "command_custom"]

#	@command_set("scheduler", per_device=True)
#	def set_scheduler(self, value, device):
#		set_new_scheduler
#
#	@command_get("scheduler")
#	def get_scheduler(self, device):
#		return current_scheduler
#
#	@command_set("foo")
#	def set_foo(self, value):
#		set_new_foo
#
#	@command_get("foo")
#	def get_foo(self):
#		return current_foo
#

def command_set(name, per_device=False, priority=0):
	def wrapper(method):
		method._command = {
			"set": True,
			"name": name,
			"per_device": per_device,
			"priority": priority,
		}
		return method

	return wrapper

def command_get(name):
	def wrapper(method):
		method._command = {
			"get": True,
			"name": name,
		}
		return method
	return wrapper

def command_custom(name, per_device=False, priority=0):
	def wrapper(method):
		method._command = {
			"custom": True,
			"name": name,
			"per_device": per_device,
			"priority": priority,
		}
		return method
	return wrapper



import unittest
import tuned.monitors.base

class MockMonitor(tuned.monitors.base.Monitor):
	@classmethod
	def _init_available_devices(cls):
		cls._available_devices = set(["a", "b"])

	@classmethod
	def update(cls):
		for device in ["a", "b"]:
			cls._load.setdefault(device, 0)
			cls._load[device] += 1

class MonitorBaseClassTestCase(unittest.TestCase):
	def test_fail_base_class_init(self):
		with self.assertRaises(NotImplementedError):
			tuned.monitors.base.Monitor()

	def test_update_fail_with_base_class(self):
		with self.assertRaises(NotImplementedError):
			tuned.monitors.base.Monitor.update()

	def test_available_devices(self):
		monitor = MockMonitor()
		devices = MockMonitor.get_available_devices()
		self.assertEqual(devices, set(["a", "b"]))
		monitor.cleanup()

	def test_registering_instances(self):
		monitor = MockMonitor()
		self.assertIn(monitor, MockMonitor.instances())
		monitor.cleanup()
		self.assertNotIn(monitor, MockMonitor.instances())

	def test_init_with_devices(self):
		monitor = MockMonitor()
		self.assertSetEqual(set(["a", "b"]), monitor.devices)
		monitor.cleanup()

		monitor = MockMonitor(["a"])
		self.assertSetEqual(set(["a"]), monitor.devices)
		monitor.cleanup()

		monitor = MockMonitor([])
		self.assertSetEqual(set(), monitor.devices)
		monitor.cleanup()

		monitor = MockMonitor(["b", "x"])
		self.assertSetEqual(set(["b"]), monitor.devices)
		monitor.cleanup()

	def test_add_device(self):
		monitor = MockMonitor(["a"])
		self.assertSetEqual(set(["a"]), monitor.devices)
		monitor.add_device("x")
		self.assertSetEqual(set(["a"]), monitor.devices)
		monitor.add_device("b")
		self.assertSetEqual(set(["a", "b"]), monitor.devices)
		monitor.cleanup()

	def test_remove_device(self):
		monitor = MockMonitor()
		self.assertSetEqual(set(["a", "b"]), monitor.devices)
		monitor.remove_device("a")
		self.assertSetEqual(set(["b"]), monitor.devices)
		monitor.remove_device("x")
		self.assertSetEqual(set(["b"]), monitor.devices)
		monitor.remove_device("b")
		self.assertSetEqual(set(), monitor.devices)
		monitor.cleanup()

	def test_get_load_from_enabled(self):
		monitor = MockMonitor()
		load = monitor.get_load()
		self.assertIn("a", load)
		self.assertIn("b", load)

		monitor.remove_device("a")
		load = monitor.get_load()
		self.assertNotIn("a", load)
		self.assertIn("b", load)

		monitor.remove_device("b")
		load = monitor.get_load()
		self.assertDictEqual({}, load)

		monitor.cleanup()

	def test_refresh_of_updating_devices(self):
		monitor1 = MockMonitor(["a"])
		self.assertSetEqual(set(["a"]), MockMonitor._updating_devices)
		monitor2 = MockMonitor(["a", "b"])
		self.assertSetEqual(set(["a", "b"]), MockMonitor._updating_devices)
		monitor1.cleanup()
		self.assertSetEqual(set(["a", "b"]), MockMonitor._updating_devices)
		monitor2.cleanup()
		self.assertSetEqual(set(), MockMonitor._updating_devices)



import unittest
from tuned.profiles import Unit

class UnitTestCase(unittest.TestCase):

	def test_default_options(self):
		unit = Unit("sample", {})
		self.assertEqual(unit.name, "sample")
		self.assertEqual(unit.type, "sample")
		self.assertTrue(unit.enabled)
		self.assertFalse(unit.replace)
		self.assertDictEqual(unit.options, {})

	def test_option_type(self):
		unit = Unit("sample", {"type": "net"})
		self.assertEqual(unit.type, "net")

	def test_option_enabled(self):
		unit = Unit("sample", {"enabled": False})
		self.assertFalse(unit.enabled)
		unit.enabled = True
		self.assertTrue(unit.enabled)

	def test_option_replace(self):
		unit = Unit("sample", {"replace": True})
		self.assertTrue(unit.replace)

	def test_option_custom(self):
		unit = Unit("sample", {"enabled": True, "type": "net", "custom": "value", "foo": "bar"})
		self.assertDictEqual(unit.options, {"custom": "value", "foo": "bar"})
		unit.options = {"hello": "world"}
		self.assertDictEqual(unit.options, {"hello": "world"})

	def test_parsing_options(self):
		unit = Unit("sample", {"type": "net", "enabled": True, "replace": True, "other": "foo"})
		self.assertEqual(unit.type, "net")



import tuned.logs
log = tuned.logs.get()

__all__ = ["Monitor"]

class Monitor(object):
	"""
	Base class for all monitors.

	Monitors provide data about the running system to Plugin objects, which use the data
	to tune system parameters.

	Following methods require reimplementation:
	  - _init_available_devices(cls)
	  - update(cls)
	"""

	# class properties

	@classmethod
	def _init_class(cls):
		cls._class_initialized = False
		cls._instances = set()
		cls._available_devices = set()
		cls._updating_devices = set()
		cls._load = {}

		cls._init_available_devices()
		assert isinstance(cls._available_devices, set)
		cls._class_initialized = True
		log.debug("available devices: %s" % ", ".join(cls._available_devices))

	@classmethod
	def _init_available_devices(cls):
		raise NotImplementedError()

	@classmethod
	def _update_available_devices(cls):
		cls._init_available_devices()
		log.debug("available devices updated to: %s"
				% ", ".join(cls._available_devices))

	@classmethod
	def get_available_devices(cls):
		return cls._available_devices

	@classmethod
	def update(cls):
		raise NotImplementedError()

	@classmethod
	def _register_instance(cls, instance):
		cls._instances.add(instance)

	@classmethod
	def _deregister_instance(cls, instance):
		cls._instances.remove(instance)

	@classmethod
	def _refresh_updating_devices(cls):
		new_updating = set()
		for instance in cls._instances:
			new_updating |= instance.devices
		cls._updating_devices.clear()
		cls._updating_devices.update(new_updating)

	@classmethod
	def instances(cls):
		return cls._instances

	# instance properties

	def __init__(self, devices = None):
		if not hasattr(self, "_class_initialized"):
			self._init_class()
			assert hasattr(self, "_class_initialized")

		self._register_instance(self)

		if devices is not None:
			self.devices = devices
		else:
			self.devices = self.get_available_devices()

		self.update()

	def __del__(self):
		try:
			self.cleanup()
		except:
			pass

	def cleanup(self):
		self._deregister_instance(self)
		self._refresh_updating_devices()

	@property
	def devices(self):
		return self._devices

	@devices.setter
	def devices(self, value):
		new_devices = self._available_devices & set(value)
		self._devices = new_devices
		self._refresh_updating_devices()

	def add_device(self, device):
		assert (isinstance(device,str) or isinstance(device,unicode))
		self._update_available_devices()
		if device in self._available_devices:
			self._devices.add(device)
			self._updating_devices.add(device)

	def remove_device(self, device):
		assert (isinstance(device,str) or isinstance(device,unicode))
		if device in self._devices:
			self._devices.remove(device)
			self._updating_devices.remove(device)

	def get_load(self):
		return dict([dev_load for dev_load in list(self._load.items()) if dev_load[0] in self._devices])

	def get_device_load(self, device):
		return self._load.get(device, None)



from . import base
from .decorators import *
import tuned.logs
from tuned.utils.commands import commands
import os
import re

log = tuned.logs.get()

class VideoPlugin(base.Plugin):
	"""
	`video`::
	
	Sets various powersave levels on video cards. Currently, only the
	Radeon cards are supported. The powersave level can be specified
	by using the [option]`radeon_powersave` option. Supported values are:
	+
	--
	* `default`
	* `auto`
	* `low`
	* `mid`
	* `high`
	* `dynpm`
	* `dpm-battery`
	* `dpm-balanced`
	* `dpm-perfomance`
	--
	+
	For additional detail, see
	link:https://www.x.org/wiki/RadeonFeature/#kmspowermanagementoptions[KMS Power Management Options].
	+
	NOTE: This plug-in is experimental and the option might change in future releases.
	+
	.To set the powersave level for the Radeon video card to high
	====
	----
	[video]
	radeon_powersave=high
	----
	====
	"""

	def _init_devices(self):
		self._devices_supported = True
		self._free_devices = set()
		self._assigned_devices = set()

		# FIXME: this is a blind shot, needs testing
		for device in self._hardware_inventory.get_devices("drm").match_sys_name("card*").match_property("DEVTYPE", "drm_minor"):
			self._free_devices.add(device.sys_name)

		self._cmd = commands()

	def _get_device_objects(self, devices):
		return [self._hardware_inventory.get_device("drm", x) for x in devices]

	@classmethod
	def _get_config_options(self):
		return {
			"radeon_powersave" : None,
		}

	def _instance_init(self, instance):
		instance._has_dynamic_tuning = False
		instance._has_static_tuning = True

	def _instance_cleanup(self, instance):
		pass

	def _radeon_powersave_files(self, device):
		return {
			"method" : "/sys/class/drm/%s/device/power_method" % device,
			"profile": "/sys/class/drm/%s/device/power_profile" % device,
			"dpm_state": "/sys/class/drm/%s/device/power_dpm_state" % device
		}

	@command_set("radeon_powersave", per_device=True)
	def _set_radeon_powersave(self, value, device, sim):
		sys_files = self._radeon_powersave_files(device)
		va = str(re.sub(r"(\s*:\s*)|(\s+)|(\s*;\s*)|(\s*,\s*)", " ", value)).split()
		if not os.path.exists(sys_files["method"]):
			if not sim:
				log.warn("radeon_powersave is not supported on '%s'" % device)
				return None
		for v in va:
			if v in ["default", "auto", "low", "mid", "high"]:
				if not sim:
					if (self._cmd.write_to_file(sys_files["method"], "profile") and
						self._cmd.write_to_file(sys_files["profile"], v)):
						return v
			elif v == "dynpm":
				if not sim:
					if (self._cmd.write_to_file(sys_files["method"], "dynpm")):
						return "dynpm"
			# new DPM profiles, recommended to use if supported
			elif v in ["dpm-battery", "dpm-balanced", "dpm-performance"]:
				if not sim:
					state = v[len("dpm-"):]
					if (self._cmd.write_to_file(sys_files["method"], "dpm") and
						self._cmd.write_to_file(sys_files["dpm_state"], state)):
						return v
			else:
				if not sim:
					log.warn("Invalid option for radeon_powersave.")
				return None
		return None

	@command_get("radeon_powersave")
	def _get_radeon_powersave(self, device, ignore_missing = False):
		sys_files = self._radeon_powersave_files(device)
		method = self._cmd.read_file(sys_files["method"], no_error=ignore_missing).strip()
		if method == "profile":
			return self._cmd.read_file(sys_files["profile"]).strip()
		elif method == "dynpm":
			return method
		elif method == "dpm":
			return "dpm-" + self._cmd.read_file(sys_files["dpm_state"]).strip()
		else:
			return None



import os
import re
import glob
from . import repository
import tuned.logs
import tuned.consts as consts
from tuned.utils.commands import commands

log = tuned.logs.get()

cmd = commands()

class Functions():
	"""
	Built-in functions
	"""

	def __init__(self):
		self._repository = repository.Repository()
		self._parse_init()

	def _parse_init(self, s = ""):
		self._cnt = 0
		self._str = s
		self._len = len(s)
		self._stack = []
		self._esc = False

	def _curr_char(self):
		return self._str[self._cnt] if self._cnt < self._len else ""

	def _curr_substr(self, _len):
		return self._str[self._cnt:self._cnt + _len]

	def _push_pos(self, esc):
		self._stack.append((esc, self._cnt))

	def _sub(self, a, b, s):
		self._str = self._str[:a] + s + self._str[b + 1:]
		self._len = len(self._str)
		self._cnt += len(s) - (b - a + 1)
		if self._cnt < 0:
			self._cnt = 0

	def _process_func(self, _from):
		sl = re.split(r'(?<!\\):', self._str[_from:self._cnt])
		if sl[0] != "${f":
			return
		sl = [str(v).replace("\:", ":") for v in sl]
		if not re.match(r'\w+$', sl[1]):
			log.error("invalid function name '%s'" % sl[1])
			return
		try:
			f = self._repository.load_func(sl[1])
		except ImportError:
			log.error("function '%s' not implemented" % sl[1])
			return
		s = f.execute(sl[2:])
		if s is None:
			return
		self._sub(_from, self._cnt, s)

	def _process(self, s):
		self._parse_init(s)
		while self._cnt < self._len:
			if self._curr_char() == "}":
				try:
					si = self._stack.pop()
				except IndexError:
					log.error("invalid variable syntax, non pair '}' in: '%s'" % s)
					return self._str
				# if not escaped
				if not si[0]:
					self._process_func(si[1])
			elif self._curr_substr(2) == "${":
				self._push_pos(self._esc)
			if self._curr_char() == "\\":
				self._esc = True
			else:
				self._esc = False
			self._cnt += 1
		if len(self._stack):
			log.error("invalid varialbe syntax, non pair '{' in: '%s'" % s)
		return self._str

	def expand(self, s):
		if s is None or s == "":
			return s
		# expand functions and convert all \${f:*} to ${f:*} (unescape)
		return re.sub(r'\\(\${f:.*})', r'\1', self._process(s))



#!/usr/bin/python3 -Es
#
# tuned: daemon for monitoring and adaptive tuning of system devices
#
# Copyright (C) 2008-2013 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

from __future__ import print_function
import argparse
import os
import sys
import traceback
import tuned.logs
import tuned.daemon
import tuned.exceptions
import tuned.consts as consts
import tuned.version as ver
from tuned.utils.global_config import GlobalConfig

def error(message):
	print(message, file=sys.stderr)

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description = "Daemon for monitoring and adaptive tuning of system devices.")
	parser.add_argument("--daemon", "-d", action = "store_true", help = "run on background")
	parser.add_argument("--debug", "-D", action = "store_true", help = "show/log debugging messages")
	parser.add_argument("--log", "-l", nargs = "?", const = consts.LOG_FILE, help = "log to file, default file: " + consts.LOG_FILE)
	parser.add_argument("--pid", "-P", nargs = "?", const = consts.PID_FILE, help = "write PID file, default file: " + consts.PID_FILE)
	parser.add_argument("--no-dbus", action = "store_true", help = "do not attach to DBus")
	parser.add_argument("--no-socket", action = "store_true", help = "do not attach to socket")
	parser.add_argument("--profile", "-p", action = "store", type=str, metavar = "name", help = "tuning profile to be activated")
	parser.add_argument('--version', "-v", action = "version", version = "%%(prog)s %s.%s.%s" % (ver.TUNED_VERSION_MAJOR, ver.TUNED_VERSION_MINOR, ver.TUNED_VERSION_PATCH))
	args = parser.parse_args(sys.argv[1:])

	if os.geteuid() != 0:
		error("Superuser permissions are required to run the daemon.")
		sys.exit(1)

	config = GlobalConfig()
	log = tuned.logs.get()
	if args.debug:
		log.setLevel("DEBUG")

	try:
		maxBytes = config.get_size("log_file_max_size", consts.LOG_FILE_MAXBYTES)
		backupCount = config.get("log_file_count", consts.LOG_FILE_COUNT)
		if args.daemon:
			if args.log is None:
				args.log = consts.LOG_FILE
			log.switch_to_file(args.log, maxBytes, backupCount)
		else:
			if args.log is not None:
				log.switch_to_file(args.log, maxBytes, backupCount)

		app = tuned.daemon.Application(args.profile, config)

		# no daemon mode doesn't need DBus or if disabled in config
		if not config.get(consts.CFG_DAEMON, consts.CFG_DEF_DAEMON) \
				or not config.get_bool(consts.CFG_ENABLE_DBUS, consts.CFG_DEF_ENABLE_DBUS):
			args.no_dbus = True

		# no daemon mode doesn't need sockets or if disabled in config
		if not config.get(consts.CFG_DAEMON, consts.CFG_DEF_DAEMON) \
				or not config.get_bool(consts.CFG_ENABLE_UNIX_SOCKET, consts.CFG_DEF_ENABLE_UNIX_SOCKET):
			args.no_socket = True

		if not args.no_dbus:
			app.attach_to_dbus(consts.DBUS_BUS, consts.DBUS_OBJECT, consts.DBUS_INTERFACE)

		if not args.no_socket:
			app.attach_to_unix_socket()

		if not args.no_dbus or not args.no_socket:
			app.register_controller()

		# always write PID file
		if args.pid is None:
			args.pid = consts.PID_FILE

		if args.daemon:
			app.daemonize(args.pid)
		else:
			app.write_pid_file(args.pid)
		app.run(args.daemon)

	except tuned.exceptions.TunedException as exception:
		if (args.debug):
			traceback.print_exc()
		else:
			error(str(exception))
			sys.exit(1)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright (C) 2014 Red Hat, Inc.
# Authors: Marek Staňa, Jaroslav Škarvada <jskarvad@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

'''
Created on Oct 15, 2013

@author: mstana
'''
from __future__ import print_function
try:
    import gi
except ImportError:
    raise ImportError("Gtk3 backend requires pygobject to be installed.")

try:
    gi.require_version("Gtk", "3.0")
except AttributeError:
    raise ImportError(
        "pygobject version too old -- it must have require_version")
except ValueError:
    raise ImportError(
        "Gtk3 backend requires the GObject introspection bindings for Gtk 3 "
        "to be installed.")

try:
    from gi.repository import Gtk, GObject
except ImportError:
    raise ImportError("Gtk3 backend requires pygobject to be installed.")

import sys
import os
import time
import collections
import subprocess

import tuned.logs
import tuned.consts as consts
import tuned.version as version
import tuned.admin.dbus_controller
import tuned.gtk.gui_profile_loader
import tuned.gtk.gui_plugin_loader
import tuned.profiles.profile as profile
import tuned.utils.global_config
from tuned.gtk.tuned_dialog import TunedDialog

from tuned.gtk.managerException import ManagerException

EXECNAME = '/usr/sbin/tuned-gui'
GLADEUI = '/usr/share/tuned/ui/tuned-gui.glade'
LICENSE = 'GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>'
NAME = 'tuned'
VERSION = 'tuned ' + str(version.TUNED_VERSION_MAJOR) + '.' + \
	str(version.TUNED_VERSION_MINOR) + '.' + str(version.TUNED_VERSION_PATCH)
COPYRIGHT = 'Copyright (C) 2014 Red Hat, Inc.'

AUTHORS = [
	'',
	'Marek Staňa',
	'Jaroslav Škarvada <jskarvad@redhat.com>',
	]

class Base(object):

	"""
	GUI class for program TuneD.
	"""

	is_admin = False

	def _starting(self):
		try:
			self.controller = \
				tuned.admin.DBusController(consts.DBUS_BUS,
					consts.DBUS_INTERFACE, consts.DBUS_OBJECT)
			self.controller.is_running()
		except tuned.admin.exceptions.TunedAdminDBusException as ex:
			response = self._gobj('tunedDaemonExceptionDialog').run()
			if response == 0:

#				 button Turn ON pressed
#				 switch_tuned_start_stop notify the switch which call funcion start_tuned

				self._start_tuned()
				self._gobj('tunedDaemonExceptionDialog').hide()
				return True
			else:
				self.error_dialog('TuneD is shutting down.',
								  'Reason: missing communication with TuneD daemon.'
								  )
				return False
		return True

	def __init__(self):

		self.active_profile = None

		self.config = tuned.utils.global_config.GlobalConfig()
		self.builder = Gtk.Builder()
		try:
			self.builder.add_from_file(GLADEUI)
		except GObject.GError as e:
			print("Error loading '%s'" % GLADEUI, file=sys.stderr)
			sys.exit(1)

		#
		#	DIALOGS
		#

		self.builder.connect_signals(self)

		if not self._starting():
			return

		self.manager = tuned.gtk.gui_profile_loader.GuiProfileLoader(
			tuned.consts.LOAD_DIRECTORIES)

		self.plugin_loader = tuned.gtk.gui_plugin_loader.GuiPluginLoader()

		self._build_about_dialog()

		#
		#	SET WIDGETS
		#

		self.treestore_profiles = Gtk.ListStore(GObject.TYPE_STRING,
				GObject.TYPE_STRING)
		self.treestore_plugins = Gtk.ListStore(GObject.TYPE_STRING)
		for plugin_name in self.plugin_loader.plugins:
			self.treestore_plugins.append([plugin_name])

		self._gobj('comboboxPlugins').set_model(self.treestore_plugins)

		self._gobj('comboboxMainPlugins').set_model(self.treestore_plugins)

		self._gobj('comboboxIncludeProfile').set_model(self.treestore_profiles)
		cell = Gtk.CellRendererText()
		self._gobj('comboboxIncludeProfile').pack_start(cell, True)
		self._gobj('comboboxIncludeProfile').add_attribute(cell, 'text', 0)

		self._gobj('treeviewProfileManager').append_column(Gtk.TreeViewColumn('Type'
				, Gtk.CellRendererText(), text=1))
		self._gobj('treeviewProfileManager').append_column(Gtk.TreeViewColumn('Name'
				, Gtk.CellRendererText(), text=0))
		self._gobj('treeviewProfileManager').set_model(self.treestore_profiles)

		self._update_profile_list()

		self._gobj('treeviewProfileManager').get_selection().select_path(0)

		self._gobj('labelActualProfile').set_text(self.controller.active_profile())
		if self.config.get(consts.CFG_RECOMMEND_COMMAND):
			self._gobj('label_recommemnded_profile').set_text(self.controller.recommend_profile())

		self.data_for_listbox_summary_of_active_profile()
		self._gobj('comboboxtextFastChangeProfile').set_model(self.treestore_profiles)
		self._gobj('labelDbusStatus').set_text(str(bool(self.controller.is_running())))

		self._gobj('switchTunedStartupStartStop').set_active(
			self.service_run_on_start_up('tuned'))

		self._gobj('switchTunedAdminFunctions').set_active(self.is_admin)

		self._gobj('comboboxtextFastChangeProfile').set_active(self.get_iter_from_model_by_name(self._gobj('comboboxtextFastChangeProfile').get_model(),
				self.controller.active_profile()))

		self.editing_profile_name = None

		# self.treeview_profile_manager.connect('row-activated',lambda x,y,z: self.execute_update_profile(x,y))
		# TO DO: need to be fixed! - double click on treeview

		self._gobj('mainWindow').show()
		Gtk.main()

	def get_iter_from_model_by_name(self, model, item_name):
		'''
		Return iter from model selected by name of item in this model
		'''

		model = self._gobj('comboboxIncludeProfile').get_model()
		selected = 0
		for item in model:
			try:
				if item[0] == item_name:
					selected = int(item.path.to_string())
			except KeyError:
				pass
		return selected

	def is_tuned_connection_ok(self):
		"""
		Result True, False depends on if tuned daemon is running. If its not running this method try to start tuned.
		"""

		try:
			self.controller.is_running()
			return True
		except tuned.admin.exceptions.TunedAdminDBusException:
			response = self._gobj('tunedDaemonExceptionDialog').run()
			if response == 0:

#				 button Turn ON pressed
#				 switch_tuned_start_stop notify the switch which call funcion start_tuned

				try:
					self._start_tuned()
					self._gobj('tunedDaemonExceptionDialog').hide()
					self._gobj('switchTunedStartStop').set_active(True)
					return True
				except:
					self._gobj('tunedDaemonExceptionDialog').hide()
					return False
			else:
				self._gobj('tunedDaemonExceptionDialog').hide()
				return False

	def data_for_listbox_summary_of_active_profile(self):
		"""
		This add rows to object listbox_summary_of_active_profile.
		Row consist of grid. Inside grid on first possition is label, second possition is vertical grid.
		label = name of plugin
		vertical grid consist of labels where are stored values for plugin option and value.

		This method is emited after change profile and on startup of app.
		"""

		for row in self._gobj('listboxSummaryOfActiveProfile'):
			self._gobj('listboxSummaryOfActiveProfile').remove(row)

		if self.is_tuned_connection_ok():
			self.active_profile = \
				self.manager.get_profile(self.controller.active_profile())
		else:
			self.active_profile = None
		self._gobj('summaryProfileName').set_text(self.active_profile.name)
		try:
			self._gobj('summaryIncludedProfileName').set_text(self.active_profile.options['include'
					])
		except:

			# keyerror probably

			self._gobj('summaryIncludedProfileName').set_text('None')

		row = Gtk.ListBoxRow()
		box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
		plugin_name = Gtk.Label()
		plugin_name.set_markup('<b>Plugin Name</b>')
		plugin_option = Gtk.Label()
		plugin_option.set_markup('<b>Plugin Options</b>')
		box.pack_start(plugin_name, True, True, 0)
		box.pack_start(plugin_option, True, True, 0)
		row.add(box)

		self._gobj('listboxSummaryOfActiveProfile').add(row)

		sep = Gtk.Separator.new(Gtk.Orientation.HORIZONTAL)
		self._gobj('listboxSummaryOfActiveProfile').add(sep)
		sep.show()

		for u in self.active_profile.units:
			row = Gtk.ListBoxRow()
			hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL,
						   spacing=0)
			hbox.set_homogeneous(True)
			row.add(hbox)
			label = Gtk.Label()
			label.set_markup(u)
			label.set_justify(Gtk.Justification.LEFT)
			hbox.pack_start(label, False, True, 1)

			grid = Gtk.Box(orientation=Gtk.Orientation.VERTICAL,
						   spacing=0)
			grid.set_homogeneous(True)
			for o in self.active_profile.units[u].options:
				label_option = Gtk.Label()
				label_option.set_markup(o + ' = ' + '<b>'
						+ self.active_profile.units[u].options[o]
						+ '</b>')
				grid.pack_start(label_option, False, True, 0)

			hbox.pack_start(grid, False, True, 0)
			self._gobj('listboxSummaryOfActiveProfile').add(row)
			separator = Gtk.Separator.new(Gtk.Orientation.HORIZONTAL)
			self._gobj('listboxSummaryOfActiveProfile').add(separator)
			separator.show()

		self._gobj('listboxSummaryOfActiveProfile').show_all()

	# def on_treeview_button_press_event(self, treeview, event):
	#	 popup = Gtk.Menu()
	#	 popup.append(Gtk.MenuItem('add'))
	#	 popup.append(Gtk.MenuItem('delete'))
	#
	#	 if event.button == 3:
	#		 x = int(event.x)
	#		 y = int(event.y)
	#		 time = event.time
	#		 pthinfo = treeview.get_path_at_pos(x, y)
	#		 if pthinfo is not None:
	#			 path, col, cellx, celly = pthinfo
	#			 treeview.grab_focus()
	#			 treeview.set_cursor(path, col, 0)
	#			 popup.popup(None, None, lambda menu, data:
	#			 (event.get_root_coords()[0], event.get_root_coords()[1], True), None, event.button, event.time)
	#		 return True

	def on_changed_combobox_plugins(self, combo):
		plugin_name = self._gobj('comboboxMainPlugins').get_active_text()
		plugin_parameters = self.plugin_loader.plugins.get(plugin_name, None)
		if plugin_parameters is None:
			self._gobj('textviewPluginAvaibleText').get_buffer().set_text('')
			self._gobj('textviewPluginDocumentationText').get_buffer().set_text(''
					)
			return
		plugin_hints = self.plugin_loader.get_plugin_hints(plugin_name)
		options = ''
		for (key, val) in plugin_parameters.items():
			options += '%s = %r\n' % (key, str(val))
			hint = plugin_hints.get(key, None)
			if hint:
				options += '%s\n' % (str(hint))

		self._gobj('textviewPluginAvaibleText').get_buffer().set_text(options)
		plugin_doc = self.plugin_loader.get_plugin_doc(plugin_name)
		self._gobj('textviewPluginDocumentationText').get_buffer().set_text(
			plugin_doc
		)

	def on_delete_event(self, window, data):
		window.hide()
		return True

	def _get_active_profile_name(self):
		return self.manager.get_profile(self.controller.active_profile()).name

	def execute_remove_profile(self, button):
		profile = self.get_treeview_selected()
		try:
			if self._get_active_profile_name() == profile:
				self.error_dialog('You can not remove active profile',
								  'Please deactivate profile by choosing another!'
								  )
				return
			if profile is None:
				self.error_dialog('No profile selected!', '')
				return
			if self._gobj('windowProfileEditor').is_visible():
				self.error_dialog('You are editing '
								  + self.editing_profile_name
								  + ' profile.',
								  'Please close edit window and try again.'
								  )
				return

			try:
				self.manager.remove_profile(profile, is_admin=self.is_admin)
			except ManagerException:
				self.error_dialog('failed to authorize', '')
				return

			for item in self.treestore_profiles:
				if item[0] == profile:
					iter = self.treestore_profiles.get_iter(item.path)
					self.treestore_profiles.remove(iter)
		except ManagerException as ex:
			self.error_dialog('Profile can not be remove', ex.__str__())

	def execute_cancel_window_profile_editor(self, button):
		self._gobj('windowProfileEditor').hide()

	def execute_cancel_window_profile_editor_raw(self, button):
		self._gobj('windowProfileEditorRaw').hide()

	def execute_open_raw_button(self, button):
		profile_name = self.get_treeview_selected()
		text_buffer = self._gobj('textviewProfileConfigRaw').get_buffer()
		text_buffer.set_text(self.manager.get_raw_profile(profile_name))
		self._gobj('windowProfileEditorRaw').show_all()

	def execute_add_plugin_to_notebook(self, button):
		if self.choose_plugin_dialog() == 1:
			plugin_name = self._gobj('comboboxPlugins').get_active_text()
			plugin_to_tab = None
			for plugin in self.plugin_loader.plugins:
				if plugin == plugin_name:
					for children in self._gobj('notebookPlugins'):
						if plugin_name \
							== self._gobj('notebookPlugins').get_menu_label_text(children):
							self.error_dialog('Plugin ' + plugin_name
									+ ' is already in profile.', '')
							return
					config_options = self.plugin_loader.plugins[plugin]
					self._gobj('notebookPlugins').append_page_menu(
							self.treeview_for_data(
								config_options, plugin),
							Gtk.Label(plugin),
							Gtk.Label(plugin)
							)

					self._gobj('notebookPlugins').show_all()

	def execute_remove_plugin_from_notebook(self, data):
		treestore = Gtk.ListStore(GObject.TYPE_STRING)
		for children in self._gobj('notebookPlugins').get_children():
			treestore.append([self._gobj('notebookPlugins').get_menu_label_text(children)])
		self._gobj('comboboxPlugins').set_model(treestore)

		response_of_dialog = self.choose_plugin_dialog()

		if response_of_dialog == 1:

#			 ok button pressed

			selected = self._gobj('comboboxPlugins').get_active_text()
			for children in self._gobj('notebookPlugins').get_children():
				if self._gobj('notebookPlugins').get_menu_label_text(children) \
					== selected:
					self._gobj('notebookPlugins').remove(children)
			self._gobj('comboboxPlugins').set_model(self.treestore_plugins)

	def execute_apply_window_profile_editor_raw(self, data):
		text_buffer = self._gobj('textviewProfileConfigRaw').get_buffer()
		start = text_buffer.get_start_iter()
		end = text_buffer.get_end_iter()
		profile_name = self.get_treeview_selected()
		try:
			self.manager.set_raw_profile(profile_name,
				text_buffer.get_text(start, end, True))
		except Exception:
			self.error_dialog('Error while parsing raw configuration')
			return

		self.error_dialog('Profile Editor will be closed.',
						  'for next updates reopen profile.')
		self._gobj('windowProfileEditor').hide()
		self._gobj('windowProfileEditorRaw').hide()

#		 refresh window_profile_editor

	def execute_create_profile(self, button):
		self.reset_values_window_edit_profile()
		self._gobj('buttonConfirmProfileCreate').show()
		self._gobj('buttonConfirmProfileUpdate').hide()
		self._gobj('buttonOpenRaw').hide()

		for child in self._gobj('notebookPlugins').get_children():
			self._gobj('notebookPlugins').remove(child)
		self._gobj('windowProfileEditor').show()

	def reset_values_window_edit_profile(self):
		self._gobj('entryProfileName').set_text('')
		self._gobj('comboboxIncludeProfile').set_active(0)
		for child in self._gobj('notebookPlugins').get_children():
			self._gobj('notebookPlugins').remove(child)

	def get_treeview_selected(self):
		"""
		Return value of treeview which is selected at calling moment of this function.
		"""

		selection = self._gobj('treeviewProfileManager').get_selection()
		(model, iter) = selection.get_selected()
		if iter is None:
			self.error_dialog('No profile selected', '')
		return self.treestore_profiles.get_value(iter, 0)

	def on_click_button_confirm_profile_update(self, data):
		profile_name = self.get_treeview_selected()
		prof = self.data_to_profile_config()
		for item in self.treestore_profiles:
			try:
				if item[0] == profile_name:
					iter = self.treestore_profiles.get_iter(item.path)
					self.treestore_profiles.remove(iter)
			except KeyError:
				raise KeyError('this cant happen')

		try:
			self.manager.update_profile(profile_name, prof, self.is_admin)
		except ManagerException:
			self.error_dialog('failed to authorize', '')
			return

		if self.manager.is_profile_factory(prof.name):
			prefix = consts.PREFIX_PROFILE_FACTORY
		else:
			prefix = consts.PREFIX_PROFILE_USER
		self.treestore_profiles.append([prof.name, prefix])
		self._gobj('windowProfileEditor').hide()

	def data_to_profile_config(self):
		name = self._gobj('entryProfileName').get_text()
		config = collections.OrderedDict()

		activated = self._gobj('comboboxIncludeProfile').get_active()
		model = self._gobj('comboboxIncludeProfile').get_model()

		include = model[activated][0]
		if self._gobj('togglebuttonIncludeProfile').get_active():
			config['main'] = {'include': include}
		for children in self._gobj('notebookPlugins'):
			acumulate_options = {}
			for item in children.get_model():
				if item[0] != 'None':
					acumulate_options[item[1]] = item[0]
			config[self._gobj('notebookPlugins').get_menu_label_text(children)] = \
				acumulate_options
		return profile.Profile(name, config)

	def on_click_button_confirm_profile_create(self, data):

		# try:

		prof = self.data_to_profile_config()
		try:
			self.manager.save_profile(prof)
		except ManagerException:
			self.error_dialog('failed to authorize', '')
			return
		self.manager._load_all_profiles()
		self.treestore_profiles.append([prof.name, consts.PREFIX_PROFILE_USER])
		self._gobj('windowProfileEditor').hide()

		# except ManagerException:
		#	 self.error_dialog("Profile with name " + prof.name
		#					   + " already exist.", "Please choose another name for profile")

	def execute_update_profile(self, data):

#		 if (self.treeview_profile_manager.get_activate_on_single_click()):
#			 print "returning"
#			 print self.treeview_profile_manager.get_activate_on_single_click()
#			 return

		self._gobj('buttonConfirmProfileCreate').hide()
		self._gobj('buttonConfirmProfileUpdate').show()
		self._gobj('buttonOpenRaw').show()
		label_update_profile = \
			self.builder.get_object('labelUpdateProfile')
		label_update_profile.set_text('Update Profile')

		for child in self._gobj('notebookPlugins').get_children():
			self._gobj('notebookPlugins').remove(child)

		self.editing_profile_name = self.get_treeview_selected()

		if self.editing_profile_name is None:
			self.error_dialog('No profile Selected',
							  'To update profile please select profile.'
							  )
			return
		if self._get_active_profile_name() == self.editing_profile_name:
			self.error_dialog('You can not update active profile',
							  'Please deactivate profile by choosing another!'
							  )
			return

		copied_profile = None

		if not self.manager.is_profile_removable(self.editing_profile_name):
			if not self.manager.get_profile(
					self.editing_profile_name + '-modified'):
				if not TunedDialog('System profile can not be modified '
							+ 'but you can create its copy',
							'create copy',
							'cancel'
							).run():
					return

				copied_profile = self.manager.get_profile(
					self.editing_profile_name)
				copied_profile.name = self.editing_profile_name + '-modified'
				try:
					self.manager.save_profile(copied_profile)
				except ManagerException:
					self.error_dialog('failed to authorize', '')
					return
			else:
				if not TunedDialog('System profile can not be modified '
							+ 'but you can use its copy',
							'open copy',
							'cancel'
							).run():
					return
				copied_profile = self.manager.get_profile(
					self.editing_profile_name + '-modified')
			self._update_profile_list()
			for row in range(len(self.treestore_profiles)):
				if self.treestore_profiles[row][0] == self.editing_profile_name + '-modified':
					self._gobj('treeviewProfileManager').get_selection().select_path(row)
					break

		profile = copied_profile or self.manager.get_profile(self.editing_profile_name)
		self._gobj('entryProfileName').set_text(profile.name)
		model = self._gobj('comboboxIncludeProfile').get_model()
		selected = 0
		self._gobj('togglebuttonIncludeProfile').set_active(False)
		for item in model:
			try:
				if item[0] == profile.options['include']:
					selected = int(item.path.to_string())
					self._gobj('togglebuttonIncludeProfile').set_active(True)
			except KeyError:
				pass

		#		 profile dont have include section

		self._gobj('comboboxIncludeProfile').set_active(selected)

		# load all values not just normal

		for (name, unit) in list(profile.units.items()):
			self._gobj('notebookPlugins').append_page_menu(self.treeview_for_data(unit.options, unit.name),
					Gtk.Label(unit.name), Gtk.Label(unit.name))
		self._gobj('notebookPlugins').show_all()
		self._gobj('windowProfileEditor').show()

	def treeview_for_data(self, data, plugin_name):
		"""
		This prepare treestore and treeview for data and return treeview
		"""

		treestore = Gtk.ListStore(GObject.TYPE_STRING,
								  GObject.TYPE_STRING)

		for (option, value) in list(data.items()):
			treestore.append([str(value), option])
		treeview = Gtk.TreeView(treestore)
		renderer = Gtk.CellRendererText()
		column_option = Gtk.TreeViewColumn('Option', renderer, text=0)
		column_value = Gtk.TreeViewColumn('Value', renderer, text=1)
		treeview.append_column(column_value)
		treeview.append_column(column_option)
		treeview.enable_grid_lines = True
		treeview.connect('row-activated', self.change_value_dialog)
		treeview.connect('button_press_event', self.on_treeview_click)
		treeview.set_property('has-tooltip',True)
		model = treeview.get_model()
		treeview.connect(
			'query-tooltip',
			lambda widget, x, y, keyboard_mode, tooltip:
				self.on_option_tooltip(widget,
					x,
					y,
					keyboard_mode,
					tooltip,
					plugin_name,
					model
					)
			)
		return treeview

	def execute_change_profile(self, button):
		"""
		Change profile in main window.
		"""

		self._gobj('spinnerFastChangeProfile').show()
		self._gobj('spinnerFastChangeProfile').start()
		if button is not None:
			text = \
				self._gobj('comboboxtextFastChangeProfile').get_active_text()
			if text is not None:
				if self.is_tuned_connection_ok():
					self.controller.switch_profile(text)
					self._gobj('labelActualProfile').set_text(self.controller.active_profile())
					self.data_for_listbox_summary_of_active_profile()
					self.active_profile = \
						self.manager.get_profile(self.controller.active_profile())
				else:
					self._gobj('labelActualProfile').set_text('')
			else:
				self.error_dialog('No profile selected', '')
		self._gobj('spinnerFastChangeProfile').stop()
		self._gobj('spinnerFastChangeProfile').hide()

	def execute_switch_tuned(self, switch, data):
		"""
		Suported switch_tuned_start_stop and switch_tuned_startup_start_stop.
		"""

		if switch == self._gobj('switchTunedStartStop'):

#			 starts or stop tuned daemon

			if self._gobj('switchTunedStartStop').get_active():
				self.is_tuned_connection_ok()
			else:
				self._su_execute(['service', 'tuned', 'stop'])
				self.error_dialog('TuneD Daemon is turned off',
								  'Support of tuned is not running.')
		elif switch == self._gobj('switchTunedStartupStartStop'):

#			 switch option for start tuned on start up

			if self._gobj('switchTunedStartupStartStop').get_active():
				self._su_execute(['systemctl', 'enable', 'tuned'])
			else:
				self._su_execute(['systemctl', 'disable', 'tuned'])
		else:
			raise NotImplementedError()

	def execute_switch_tuned_admin_functions(self, switch, data):
		self.is_admin = self._gobj('switchTunedAdminFunctions').get_active()

	def service_run_on_start_up(self, service):
		"""
		Depends on if tuned is set to run on startup of system return true if yes, else return false
		"""

		temp = self._execute(['systemctl', 'is-enabled', service, '-q'])
		if temp == 0:
			return True
		return False

	def error_dialog(self, error, info):
		"""
		General error dialog with two fields. Primary and secondary text fields.
		"""

		self._gobj('messagedialogOperationError').set_markup(error)
		self._gobj('messagedialogOperationError').format_secondary_text(info)
		self._gobj('messagedialogOperationError').run()
		self._gobj('messagedialogOperationError').hide()

	def execute_about(self, widget):
		self.about_dialog.run()
		self.about_dialog.hide()

	def change_value_dialog(
		self,
		tree_view,
		path,
		treeview_column,
		):
		"""
		Shows up dialog after double click on treeview which has to be stored in notebook of plugins.
		Th``` dialog allows you to chagne specific option's value in plugin.
		"""

		model = tree_view.get_model()
		dialog = self.builder.get_object('changeValueDialog')
		button_apply = self.builder.get_object('buttonApplyChangeValue')
		button_cancel = self.builder.get_object('buttonCancel1')
		entry1 = self.builder.get_object('entry1')
		text = self.builder.get_object('labelTextDialogChangeValue')

		text.set_text(model.get_value(model.get_iter(path), 1))
		text = model.get_value(model.get_iter(path), 0)
		if text is not None:
			entry1.set_text(text)
		else:
			entry1.set_text('')
		dialog.connect('destroy', lambda d: dialog.hide())
		button_cancel.connect('clicked', lambda d: dialog.hide())

		if dialog.run() == 1:
			model.set_value(model.get_iter(path), 0, entry1.get_text())
		dialog.hide()

	def choose_plugin_dialog(self):
		"""
		Shows up dialog with combobox where are stored plugins available to add.
		"""

		self._gobj('comboboxPlugins').set_active(0)
		self.button_add_plugin_dialog = \
			self.builder.get_object('buttonAddPluginDialog')
		self.button_cancel_add_plugin_dialog = \
			self.builder.get_object('buttonCloseAddPlugin')
		self.button_cancel_add_plugin_dialog.connect('clicked',
				lambda d: self._gobj('dialogAddPlugin').hide())
		self._gobj('dialogAddPlugin').connect('destroy', lambda d: \
				self._gobj('dialogAddPlugin').hide())
		response = self._gobj('dialogAddPlugin').run()
		self._gobj('dialogAddPlugin').hide()
		return response

	def on_treeview_click(self, treeview, event):

		if event.button == 3:
			popup = Gtk.Menu()
			popup.append(Gtk.MenuItem('add'))
			popup.append(Gtk.MenuItem('delete'))
			time = event.time
			self._gobj('menuAddPluginValue').popup(
				None,
				None,
				None,
				None,
				event.button,
				time,
				)
			return True

	@staticmethod
	def liststore_contains_item(liststore, item):
		for liststore_item in liststore:
			if liststore_item[1] == item:
				return True
		return False

	def _start_tuned(self):
		self._su_execute(['service', 'tuned', 'start'])
		time.sleep(10)
		self.controller = tuned.admin.DBusController(consts.DBUS_BUS,
				consts.DBUS_INTERFACE, consts.DBUS_OBJECT)

	def _gobj(self, id):
		""" Wrapper for self.builder.get_object
		"""
		return self.builder.get_object(id)

	def HideTunedDaemonExceptionDialog(self, sender):
		self._gobj('tunedDaemonExceptionDialog').hide()

	def _build_about_dialog(self):
		self.about_dialog = Gtk.AboutDialog.new()
		self.about_dialog.set_name(NAME)
		self.about_dialog.set_version(VERSION)
		self.about_dialog.set_license(LICENSE)
		self.about_dialog.set_wrap_license(True)
		self.about_dialog.set_copyright(COPYRIGHT)
		self.about_dialog.set_authors(AUTHORS)

	def _update_profile_list(self):
		self.treestore_profiles.clear()

		for profile_name in self.manager.get_names():
			if self.manager.is_profile_factory(profile_name):
				self.treestore_profiles.append([profile_name,
						consts.PREFIX_PROFILE_FACTORY])
			else:
				self.treestore_profiles.append([profile_name,
						consts.PREFIX_PROFILE_USER])
		self._gobj('comboboxIncludeProfile').set_model(self.treestore_profiles)
		self._gobj('comboboxtextFastChangeProfile').set_model(
			self.treestore_profiles)
		self._gobj('treeviewProfileManager').set_model(self.treestore_profiles)
		self._gobj('comboboxtextFastChangeProfile').set_active(
			self.get_iter_from_model_by_name(
				self._gobj('comboboxtextFastChangeProfile').get_model(),
					self.controller.active_profile()))

	def _su_execute(self, args):
		args = ['pkexec'] + args
		rc = subprocess.call(args)
		return rc

	def _execute(self, args):
		rc = subprocess.call(args)
		return rc

	def on_option_tooltip(self, widget, x, y, keyboard_mode, tooltip, plugin_name, model):
		path = widget.get_path_at_pos(x, y)
		plugin_hints = self.plugin_loader.get_plugin_hints(plugin_name)
		if plugin_hints is None or len(plugin_hints) == 0:
			return False
		if not path:
			row_count = model.iter_n_children(None)
			if (row_count == 0):
				return False
			iterator = model.get_iter(row_count - 1)
			option_name = model.get_value(iterator, 1)
		elif int(str(path[0])) < 1:
			return False
		else:
			iterator = model.get_iter(int(str(path[0])) -1)
			option_name = model.get_value(iterator, 1)
		path = model.get_path(iterator)
		hint = plugin_hints.get(option_name, None)
		if not hint:
			return False
		tooltip.set_text(hint)
		widget.set_tooltip_row(tooltip, path)
		return True

	def gtk_main_quit(self, sender):
		Gtk.main_quit()


if __name__ == '__main__':

	base = Base()



import os
import tuned.logs
from . import base
from tuned.utils.commands import commands

log = tuned.logs.get()

class cpulist_invert(base.Function):
	"""
	Inverts list of CPUs (makes its complement). For the complement it
	gets number of online CPUs from the /sys/devices/system/cpu/online,
	e.g. system with 4 CPUs (0-3), the inversion of list "0,2,3" will be
	"1"
	"""
	def __init__(self):
		# arbitrary number of arguments
		super(cpulist_invert, self).__init__("cpulist_invert", 0)

	def execute(self, args):
		if not super(cpulist_invert, self).execute(args):
			return None
		return ",".join(str(v) for v in self._cmd.cpulist_invert(",,".join(args)))



# -*- coding: utf-8 -*-

# Copyright (C) 2008-2014 Red Hat, Inc.
# Authors: Marek Staňa, Jaroslav Škarvada <jskarvad@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

'''
Created on Apr 6, 2014

@author: mstana
'''


class ManagerException(Exception):

    """
    """

    def __init__(self, code):
        self.code = code

    def __str__(self):

        return repr(self.code)

    def profile_already_exists(self, text=None):
        if text is None:
            return repr(self.code)
        else:
            return repr(text)



# -*- coding: utf-8 -*-

# Copyright (C) 2008-2014 Red Hat, Inc.
# Authors: Marek Staňa, Jaroslav Škarvada <jskarvad@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

'''
Created on Mar 30, 2014

@author: mstana
'''

import importlib

import tuned.consts as consts
import tuned.logs
from tuned.utils.config_parser import ConfigParser, Error
from tuned.exceptions import TunedException
from tuned.utils.global_config import GlobalConfig

from tuned.admin.dbus_controller import DBusController

__all__ = ['GuiPluginLoader']


class GuiPluginLoader():

    '''
    Class for scan, import and load actual avaible plugins.
    '''

    def __init__(self):
        '''
        Constructor
        '''

        self._plugins = {}
        self.plugins_doc = {}
        self._prefix = 'plugin_'
        self._sufix = '.py'
        self._dbus_controller = DBusController(consts.DBUS_BUS,
			consts.DBUS_INTERFACE, consts.DBUS_OBJECT
            )
        self._get_plugins()

    @property
    def plugins(self):
        return self._plugins

    def _get_plugins(self):
        self._plugins = self._dbus_controller.get_plugins()

    def get_plugin_doc(self, plugin_name):
        return self._dbus_controller.get_plugin_documentation(plugin_name)

    def get_plugin_hints(self, plugin_name):
        return self._dbus_controller.get_plugin_hints(plugin_name)

    def _load_global_config(self, file_name=consts.GLOBAL_CONFIG_FILE):
        """
        Loads global configuration file.
        """

        try:
            config_parser = ConfigParser(delimiters=('='), inline_comment_prefixes=('#'), strict=False)
            config_parser.optionxform = str
            with open(file_name) as f:
                config_parser.read_string("[" + consts.MAGIC_HEADER_NAME + "]\n" + f.read(), file_name)
            config, functions = GlobalConfig.get_global_config_spec()
            for option in config_parser.options(consts.MAGIC_HEADER_NAME):
                if option in config:
                    try:
                        func = getattr(config_parser, functions[option])
                        config[option] = func(consts.MAGIC_HEADER_NAME, option)
                    except Error:
                        raise TunedException("Global TuneD configuration file '%s' is not valid."
                                             % file_name)
                else:
                    config[option] = config_parser.get(consts.MAGIC_HEADER_NAME, option, raw=True)
        except IOError as e:
            raise TunedException("Global TuneD configuration file '%s' not found."
                                  % file_name)
        except Error as e:
            raise TunedException("Error parsing global TuneD configuration file '%s'."
                                  % file_name)
        return config




import os
import tuned.logs
from . import base
from tuned.utils.commands import commands

log = tuned.logs.get()

class cpulist_pack(base.Function):
	"""
	Conversion function: packs CPU list in form 1,2,3,5 to 1-3,5.
	The cpulist_unpack is used as a preprocessor, so it always returns
	optimal results. For details about input syntax see cpulist_unpack.
	"""
	def __init__(self):
		# arbitrary number of arguments
		super(cpulist_pack, self).__init__("cpulist_pack", 0)

	def execute(self, args):
		if not super(cpulist_pack, self).execute(args):
			return None
		return ",".join(str(v) for v in self._cmd.cpulist_pack(",,".join(args)))



import unittest
import tempfile
import shutil
import os

import tuned.consts as consts
import tuned.utils.global_config as global_config

class GlobalConfigTestCase(unittest.TestCase):
	@classmethod
	def setUpClass(cls):
		cls.test_dir = tempfile.mkdtemp()
		with open(cls.test_dir + '/test_config','w') as f:
			f.write('test_option = hello #this is comment\ntest_bool = 1\ntest_size = 12MB\n'\
				+ '/sys/bus/pci/devices/0000:00:02.0/power/control=auto\n'\
				+ '/sys/bus/pci/devices/0000:04:00.0/power/control=auto\n'\
				+ 'false_bool=0\n'\
				+ consts.CFG_LOG_FILE_COUNT + " = " + str(consts.CFG_DEF_LOG_FILE_COUNT) + "1\n")

		cls._global_config = global_config.GlobalConfig(\
			cls.test_dir + '/test_config')

	def test_get(self):
		self.assertEqual(self._global_config.get('test_option'), 'hello')
		self.assertEqual(self._global_config.get('/sys/bus/pci/devices/0000:00:02.0/power/control'), 'auto')

	def test_get_bool(self):
		self.assertTrue(self._global_config.get_bool('test_bool'))
		self.assertFalse(self._global_config.get_bool('false_bool'))

	def test_get_size(self):
		self.assertEqual(self._global_config.get_size('test_size'),\
			12*1024*1024)

		self._global_config.set('test_size', 'bad_value')

		self.assertIsNone(self._global_config.get_size('test_size'))

	def test_default(self):
		daemon = self._global_config.get(consts.CFG_DAEMON)
		self.assertEqual(daemon, consts.CFG_DEF_DAEMON)

		log_file_count = self._global_config.get(consts.CFG_LOG_FILE_COUNT)
		self.assertIsNotNone(log_file_count)
		self.assertNotEqual(log_file_count, consts.CFG_DEF_LOG_FILE_COUNT)

	@classmethod
	def tearDownClass(cls):
		shutil.rmtree(cls.test_dir)



class Storage(object):
	__slots__ = ["_storage_provider", "_namespace"]

	def __init__(self, storage_provider, namespace):
		self._storage_provider = storage_provider
		self._namespace = namespace

	def set(self, option, value):
		self._storage_provider.set(self._namespace, option, value)

	def get(self, option, default=None):
		return self._storage_provider.get(self._namespace, option, default)

	def unset(self, option):
		self._storage_provider.unset(self._namespace, option)



from . import base
import glob
import re
import os.path
from .decorators import *
import tuned.logs
from subprocess import *
from tuned.utils.commands import commands

log = tuned.logs.get()

class SysfsPlugin(base.Plugin):
	"""
	`sysfs`::
	
	Sets various `sysfs` settings specified by the plug-in options.
	+
	The syntax is `_name_=_value_`, where
	`_name_` is the `sysfs` path to use and `_value_` is
	the value to write. The `sysfs` path supports the shell-style
	wildcard characters (see `man 7 glob` for additional detail).
	+
	Use this plugin in case you need to change some settings that are
	not covered by other plug-ins. Prefer specific plug-ins if they
	cover the required settings.
	+
	.Ignore corrected errors and associated scans that cause latency spikes
	====
	----
	[sysfs]
	/sys/devices/system/machinecheck/machinecheck*/ignore_ce=1
	----
	====
	"""

	# TODO: resolve possible conflicts with sysctl settings from other plugins

	def __init__(self, *args, **kwargs):
		super(SysfsPlugin, self).__init__(*args, **kwargs)
		self._has_dynamic_options = True
		self._cmd = commands()

	def _instance_init(self, instance):
		instance._has_dynamic_tuning = False
		instance._has_static_tuning = True

		instance._sysfs = dict([(os.path.normpath(key_value[0]), key_value[1]) for key_value in list(instance.options.items())])
		instance._sysfs_original = {}

	def _instance_cleanup(self, instance):
		pass

	def _instance_apply_static(self, instance):
		for key, value in list(instance._sysfs.items()):
			v = self._variables.expand(value)
			for f in glob.iglob(key):
				if self._check_sysfs(f):
					instance._sysfs_original[f] = self._read_sysfs(f)
					self._write_sysfs(f, v)
				else:
					log.error("rejecting write to '%s' (not inside /sys)" % f)

	def _instance_verify_static(self, instance, ignore_missing, devices):
		ret = True
		for key, value in list(instance._sysfs.items()):
			v = self._variables.expand(value)
			for f in glob.iglob(key):
				if self._check_sysfs(f):
					curr_val = self._read_sysfs(f)
					if self._verify_value(f, v, curr_val, ignore_missing) == False:
						ret = False
		return ret

	def _instance_unapply_static(self, instance, full_rollback = False):
		for key, value in list(instance._sysfs_original.items()):
			self._write_sysfs(key, value)

	def _check_sysfs(self, sysfs_file):
		return re.match(r"^/sys/.*", sysfs_file)

	def _read_sysfs(self, sysfs_file):
		data = self._cmd.read_file(sysfs_file).strip()
		if len(data) > 0:
			return self._cmd.get_active_option(data, False)
		else:
			return None

	def _write_sysfs(self, sysfs_file, value):
		return self._cmd.write_to_file(sysfs_file, value)



from tuned.profiles.locator import *
from tuned.profiles.loader import *
from tuned.profiles.profile import *
from tuned.profiles.unit import *
from tuned.profiles.exceptions import *
from tuned.profiles.factory import *
from tuned.profiles.merger import *
from . import functions



import unittest
import pyudev

from tuned.hardware.device_matcher_udev import DeviceMatcherUdev

class DeviceMatcherUdevTestCase(unittest.TestCase):
	@classmethod
	def setUpClass(cls):
		cls.udev_context = pyudev.Context()
		cls.matcher = DeviceMatcherUdev()

	def test_simple_search(self):
		try:
			device = pyudev.Devices.from_sys_path(self.udev_context,
				"/sys/devices/virtual/tty/tty0")
		except AttributeError:
			device = pyudev.Device.from_sys_path(self.udev_context,
				"/sys/devices/virtual/tty/tty0")
		self.assertTrue(self.matcher.match("tty0", device))
		try:
			device = pyudev.Devices.from_sys_path(self.udev_context,
				"/sys/devices/virtual/tty/tty1")
		except AttributeError:
			device = pyudev.Device.from_sys_path(self.udev_context,
				"/sys/devices/virtual/tty/tty1")
		self.assertFalse(self.matcher.match("tty0", device))

	def test_regex_search(self):
		try:
			device = pyudev.Devices.from_sys_path(self.udev_context,
				"/sys/devices/virtual/tty/tty0")
		except AttributeError:
			device = pyudev.Device.from_sys_path(self.udev_context,
				"/sys/devices/virtual/tty/tty0")
		self.assertTrue(self.matcher.match("tty.", device))
		self.assertFalse(self.matcher.match("tty[1-9]", device))



class Singleton(object):
	"""
	Singleton design pattern.
	"""

	_instance = None

	def __init__(self):
		if self.__class__ is Singleton:
			raise TypeError("Cannot instantiate directly.")

	@classmethod
	def get_instance(cls):
		"""Get the class instance."""
		if cls._instance is None:
			cls._instance = cls()
		return cls._instance



import errno
from . import hotplug
from .decorators import *
import tuned.logs
import tuned.consts as consts
from tuned.utils.commands import commands
import os
import re

log = tuned.logs.get()

class SCSIHostPlugin(hotplug.Plugin):
	"""
	`scsi_host`::
	
	Tunes options for SCSI hosts.
	+
	The plug-in sets Aggressive Link Power Management (ALPM) to the value specified
	by the [option]`alpm` option. The option takes one of three values:
	`min_power`, `medium_power` and `max_performance`.
	+
	NOTE: ALPM is only available on SATA controllers that use the Advanced
	Host Controller Interface (AHCI).
	+
	.ALPM setting when extended periods of idle time are expected
	====
	----
	[scsi_host]
	alpm=min_power
	----
	====
	"""

	def __init__(self, *args, **kwargs):
		super(SCSIHostPlugin, self).__init__(*args, **kwargs)

		self._cmd = commands()

	def _init_devices(self):
		super(SCSIHostPlugin, self)._init_devices()
		self._devices_supported = True
		self._free_devices = set()
		for device in self._hardware_inventory.get_devices("scsi"):
			if self._device_is_supported(device):
				self._free_devices.add(device.sys_name)

		self._assigned_devices = set()

	def _get_device_objects(self, devices):
		return [self._hardware_inventory.get_device("scsi", x) for x in devices]

	@classmethod
	def _device_is_supported(cls, device):
		return  device.device_type == "scsi_host"

	def _hardware_events_init(self):
		self._hardware_inventory.subscribe(self, "scsi", self._hardware_events_callback)

	def _hardware_events_cleanup(self):
		self._hardware_inventory.unsubscribe(self)

	def _hardware_events_callback(self, event, device):
		if self._device_is_supported(device):
			super(SCSIHostPlugin, self)._hardware_events_callback(event, device)

	def _added_device_apply_tuning(self, instance, device_name):
		super(SCSIHostPlugin, self)._added_device_apply_tuning(instance, device_name)

	def _removed_device_unapply_tuning(self, instance, device_name):
		super(SCSIHostPlugin, self)._removed_device_unapply_tuning(instance, device_name)

	@classmethod
	def _get_config_options(cls):
		return {
			"alpm"               : None,
		}

	def _instance_init(self, instance):
		instance._has_static_tuning = True
		instance._has_dynamic_tuning = False

	def _instance_cleanup(self, instance):
		pass

	def _get_alpm_policy_file(self, device):
		return os.path.join("/sys/class/scsi_host/", str(device), "link_power_management_policy")

	@command_set("alpm", per_device = True)
	def _set_alpm(self, policy, device, sim):
		if policy is None:
			return None
		policy_file = self._get_alpm_policy_file(device)
		if not sim:
			if os.path.exists(policy_file):
				self._cmd.write_to_file(policy_file, policy)
			else:
				log.info("ALPM control file ('%s') not found, skipping ALPM setting for '%s'" % (policy_file, str(device)))
				return None
		return policy

	@command_get("alpm")
	def _get_alpm(self, device, ignore_missing=False):
		policy_file = self._get_alpm_policy_file(device)
		policy = self._cmd.read_file(policy_file, no_error = True).strip()
		return policy if policy != "" else None



from . import base
from . import exceptions
import tuned.logs
from tuned.utils.commands import commands
import os

log = tuned.logs.get()

class EeePCSHEPlugin(base.Plugin):
	"""
	`eeepc_she`::
	
	Dynamically sets the front-side bus (FSB) speed according to the
	CPU load. This feature can be found on some netbooks and is also
	known as the Asus Super Hybrid Engine. If the CPU load is lower or
	equal to the value specified by the [option]`load_threshold_powersave`
	option, the plug-in sets the FSB speed to the value specified by the
	[option]`she_powersave` option. If the CPU load is higher or
	equal to the value specified by the [option]`load_threshold_normal`
	option, it sets the FSB speed to the value specified by the
	[option]`she_normal` option. Static tuning is not supported and the
	plug-in is transparently disabled if the hardware support for this
	feature is not detected.
	
	NOTE: For details about the FSB frequencies and corresponding values, see
	link:https://www.kernel.org/doc/Documentation/ABI/testing/sysfs-platform-eeepc-laptop[the kernel documentation].
	The provided defaults should work for most users.
	"""

	def __init__(self, *args, **kwargs):
		self._cmd = commands()
		self._control_file = "/sys/devices/platform/eeepc/cpufv"
		if not os.path.isfile(self._control_file):
			self._control_file = "/sys/devices/platform/eeepc-wmi/cpufv"
		if not os.path.isfile(self._control_file):
			raise exceptions.NotSupportedPluginException("Plugin is not supported on your hardware.")
		super(EeePCSHEPlugin, self).__init__(*args, **kwargs)

	@classmethod
	def _get_config_options(self):
		return {
			"load_threshold_normal"    : 0.6,
			"load_threshold_powersave" : 0.4,
			"she_powersave"            : 2,
			"she_normal"               : 1,
		}

	def _instance_init(self, instance):
		instance._has_static_tuning = False
		instance._has_dynamic_tuning = True
		instance._she_mode = None
		instance._load_monitor = self._monitors_repository.create("load", None)

	def _instance_cleanup(self, instance):
		if instance._load_monitor is not None:
			self._monitors_repository.delete(instance._load_monitor)
			instance._load_monitor = None

	def _instance_update_dynamic(self, instance, device):
		load = instance._load_monitor.get_load()["system"]
		if load <= instance.options["load_threshold_powersave"]:
			self._set_she_mode(instance, "powersave")
		elif load >= instance.options["load_threshold_normal"]:
			self._set_she_mode(instance, "normal")

	def _instance_unapply_dynamic(self, instance, device):
		# FIXME: restore previous value
		self._set_she_mode(instance, "normal")

	def _set_she_mode(self, instance, new_mode):
		new_mode_numeric = int(instance.options["she_%s" % new_mode])
		if instance._she_mode != new_mode_numeric:
			log.info("new eeepc_she mode %s (%d) " % (new_mode, new_mode_numeric))
			self._cmd.write_to_file(self._control_file, "%s" % new_mode_numeric)
			self._she_mode = new_mode_numeric



import unittest
try:
	from unittest.mock import Mock
	from unittest.mock import call
except ImportError:
	from mock import Mock
	from mock import call
import tuned.storage

class StorageStorageTestCase(unittest.TestCase):
	def test_set(self):
		mock_provider = Mock()
		factory = tuned.storage.Factory(mock_provider)
		storage = factory.create("foo")

		storage.set("optname", "optval")
		mock_provider.set.assert_called_once_with("foo", "optname", "optval")

	def test_get(self):
		mock_provider = Mock()
		mock_provider.get.side_effect = [ None, "defval", "somevalue" ]
		factory = tuned.storage.Factory(mock_provider)
		storage = factory.create("foo")

		self.assertEqual(None, storage.get("optname"))
		self.assertEqual("defval", storage.get("optname", "defval"))
		self.assertEqual("somevalue", storage.get("existing"))

		calls = [
				 call("foo", "optname", None),
				 call("foo", "optname", "defval"),
				 call("foo", "existing", None)
				]

		mock_provider.get.assert_has_calls(calls)



class Instance(object):
	"""
	"""

	def __init__(self, plugin, name, devices_expression, devices_udev_regex, script_pre, script_post, options):
		self._plugin = plugin
		self._name = name
		self._devices_expression = devices_expression
		self._devices_udev_regex = devices_udev_regex
		self._script_pre = script_pre
		self._script_post = script_post
		self._options = options

		self._active = True
		self._has_static_tuning = False
		self._has_dynamic_tuning = False
		self._assigned_devices = set()
		self._processed_devices = set()

	# properties

	@property
	def plugin(self):
		return self._plugin

	@property
	def name(self):
		return self._name

	@property
	def active(self):
		"""The instance performs some tuning (otherwise it is suspended)."""
		return self._active

	@active.setter
	def active(self, value):
		self._active = value

	@property
	def devices_expression(self):
		return self._devices_expression

	@property
	def assigned_devices(self):
		return self._assigned_devices

	@property
	def processed_devices(self):
		return self._processed_devices

	@property
	def devices_udev_regex(self):
		return self._devices_udev_regex

	@property
	def script_pre(self):
		return self._script_pre

	@property
	def script_post(self):
		return self._script_post

	@property
	def options(self):
		return self._options

	@property
	def has_static_tuning(self):
		return self._has_static_tuning

	@property
	def has_dynamic_tuning(self):
		return self._has_dynamic_tuning

	# methods

	def apply_tuning(self):
		self._plugin.instance_apply_tuning(self)

	def verify_tuning(self, ignore_missing):
		return self._plugin.instance_verify_tuning(self, ignore_missing)

	def update_tuning(self):
		self._plugin.instance_update_tuning(self)

	def unapply_tuning(self, full_rollback = False):
		self._plugin.instance_unapply_tuning(self, full_rollback)

	def destroy(self):
		self.unapply_tuning()
		self._plugin.destroy_instance(self)



from tuned.utils.plugin_loader import PluginLoader
import tuned.plugins.base
import tuned.logs

log = tuned.logs.get()

__all__ = ["Repository"]

class Repository(PluginLoader):

	def __init__(self, monitor_repository, storage_factory, hardware_inventory, device_matcher, device_matcher_udev, plugin_instance_factory, global_cfg, variables):
		super(Repository, self).__init__()
		self._plugins = set()
		self._monitor_repository = monitor_repository
		self._storage_factory = storage_factory
		self._hardware_inventory = hardware_inventory
		self._device_matcher = device_matcher
		self._device_matcher_udev = device_matcher_udev
		self._plugin_instance_factory = plugin_instance_factory
		self._global_cfg = global_cfg
		self._variables = variables

	@property
	def plugins(self):
		return self._plugins

	def _set_loader_parameters(self):
		self._namespace = "tuned.plugins"
		self._prefix = "plugin_"
		self._interface = tuned.plugins.base.Plugin

	def create(self, plugin_name):
		log.debug("creating plugin %s" % plugin_name)
		plugin_cls = self.load_plugin(plugin_name)
		plugin_instance = plugin_cls(self._monitor_repository, self._storage_factory, self._hardware_inventory, self._device_matcher,\
			self._device_matcher_udev, self._plugin_instance_factory, self._global_cfg, self._variables)
		self._plugins.add(plugin_instance)
		return plugin_instance

	def delete(self, plugin):
		assert isinstance(plugin, self._interface)
		log.debug("removing plugin %s" % plugin)
		plugin.cleanup()
		self._plugins.remove(plugin)



import os
import re
import pwd, grp

from . import interfaces
import tuned.logs
import tuned.consts as consts
from inspect import ismethod
import socket
import json
import select

log = tuned.logs.get()

class UnixSocketExporter(interfaces.ExporterInterface):
	"""
	Export method calls through Unix Domain Socket Interface.

	We take a method to be exported and create a simple wrapper function
	to call it. This is required as we need the original function to be
	bound to the original object instance. While the wrapper will be bound
	to an object we dynamically construct.
	"""

	def __init__(self, socket_path=consts.CFG_DEF_UNIX_SOCKET_PATH,
				 signal_paths=consts.CFG_DEF_UNIX_SOCKET_SIGNAL_PATHS,
				 ownership=consts.CFG_DEF_UNIX_SOCKET_OWNERSHIP,
				 permissions=consts.CFG_DEF_UNIX_SOCKET_PERMISIONS,
				 connections_backlog=consts.CFG_DEF_UNIX_SOCKET_CONNECTIONS_BACKLOG):

		self._socket_path = socket_path
		self._socket_object = None
		self._socket_signal_paths = re.split(r",;", signal_paths) if signal_paths else []
		self._socket_signal_objects = []
		self._ownership = [-1, -1]
		if ownership:
			ownership = ownership.split()
			for i, o in enumerate(ownership[:2]):
				try:
					self._ownership[i] = int(o)
				except ValueError:
					try:
						# user
						if i == 0:
							self._ownership[i] = pwd.getpwnam(o).pw_uid
						# group
						else:
							self._ownership[i] = grp.getgrnam(o).gr_gid
					except KeyError:
						log.error("%s '%s' does not exists, leaving default" % ("User" if i == 0 else "Group", o))
		self._permissions = permissions
		self._connections_backlog = connections_backlog

		self._unix_socket_methods = {}
		self._signals = set()
		self._conn = None
		self._channel = None

	def running(self):
		return self._socket_object is not None

	def export(self, method, in_signature, out_signature):
		if not ismethod(method):
			raise Exception("Only bound methods can be exported.")

		method_name = method.__name__
		if method_name in self._unix_socket_methods:
			raise Exception("Method with this name (%s) is already exported." % method_name)

		class wrapper(object):
			def __init__(self, in_signature, out_signature):
				self._in_signature = in_signature
				self._out_signature = out_signature
				
			def __call__(self, *args, **kwargs):
				return method(*args, **kwargs)

		self._unix_socket_methods[method_name] = wrapper(in_signature, out_signature)

	def signal(self, method, out_signature):
		if not ismethod(method):
			raise Exception("Only bound methods can be exported.")
		
		method_name = method.__name__
		if method_name in self._unix_socket_methods:
			raise Exception("Method with this name (%s) is already exported." % method_name)
		
		class wrapper(object):
			def __init__(self, out_signature):
				self._out_signature = out_signature
			
			def __call__(self, *args, **kwargs):
				return method(*args, **kwargs)
		
		self._unix_socket_methods[method_name] = wrapper(out_signature)
		self._signals.add(method_name)

	def send_signal(self, signal, *args, **kwargs):
		if not signal in self._signals:
			raise Exception("Signal '%s' doesn't exist." % signal)
		for p in self._socket_signal_paths:
			log.debug("Sending signal on socket %s" % p)
			try:
				s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
				s.setblocking(False)
				s.connect(p)
				self._send_data(s, {"jsonrpc": "2.0", "method": signal, "params": args})
				s.close()
			except OSError as e:
				log.warning("Error while sending signal '%s' to socket '%s': %s" % (signal, p, e))

	def register_signal_path(self, path):
		self._socket_signal_paths.append(path)

	def _construct_socket_object(self):
		if self._socket_path:
			if os.path.exists(self._socket_path):
				os.unlink(self._socket_path)
			self._socket_object = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
			self._socket_object.bind(self._socket_path)
			self._socket_object.listen(self._connections_backlog)
			os.chown(self._socket_path, self._ownership[0], self._ownership[1])
			if self._permissions:
				os.chmod(self._socket_path, self._permissions)

	def start(self):
		if self.running():
			return

		self.stop()
		self._construct_socket_object()

	def stop(self):
		if self._socket_object:
			self._socket_object.close()

	def _send_data(self, s, data):
		log.debug("Sending socket data: %s)" % data)
		try:
			s.send(json.dumps(data).encode("utf-8"))
		except Exception as e:
			log.warning("Failed to send data '%s': %s" % (data, e))

	def _create_response(self, data, id, error=False):
		res = {
			"jsonrpc": "2.0",
			"id": id
		}
		if error:
			res["error"] = data
		else:
			res["result"] = data
		return res

	def _create_error_responce(self, code, message, id=None, data=None):
		return self._create_response({
			"code": code,
			"message": message,
			"data": data,
		}, error=True, id=id)

	def _create_result_response(self, result, id):
		return self._create_response(result, id)

	def _check_id(self, data):
		if data.get("id"):
			return data
		return None

	def _process_request(self, req):
		if type(req) != dict or req.get("jsonrpc") != "2.0" or not req.get("method"):
			return self._create_error_responce(-32600, "Invalid Request")
		id = req.get("id")
		ret = None
		if req["method"] not in self._unix_socket_methods:
			return self._check_id(self._create_error_responce(-32601, "Method not found", id))
		try:
			if not req.get("params"):
				ret = self._unix_socket_methods[req["method"]]()
			elif type(req["params"]) in (list, tuple):
				ret = self._unix_socket_methods[req["method"]](*req["params"])
			elif type(req["params"]) == dict:
				ret = self._unix_socket_methods[req["method"]](**req["params"])
			else:
				return self._check_id(self._create_error_responce(-32600, "Invalid Request", id))
		except TypeError as e:
			return self._check_id(self._create_error_responce(-32602, "Invalid params", id, str(e)))
		except Exception as e:
			return self._check_id(self._create_error_responce(1, "Error", id, str(e)))
		return self._check_id(self._create_result_response(ret, id))

	def period_check(self):
		"""
		Periodically checks socket object for new calls. This allows to function without special thread.
		Interface is according JSON-RPC 2.0 Specification (see https://www.jsonrpc.org/specification)
		
		Example calls:
		
		printf '[{"jsonrpc": "2.0", "method": "active_profile", "id": 1}, {"jsonrpc": "2.0", "method": "profiles", "id": 2}]' | nc -U /run/tuned/tuned.sock
		printf '{"jsonrpc": "2.0", "method": "switch_profile", "params": {"profile_name": "balanced"}, "id": 1}' | nc -U /run/tuned/tuned.sock
		"""
		if not self.running():
			return
		while True:
			r, _, _ = select.select([self._socket_object], (), (), 0)
			if r:
				conn, _ = self._socket_object.accept()
				try:
					data = ""
					while True:
						rec_data = conn.recv(4096).decode()
						if not rec_data:
							break
						data += rec_data
				except Exception as e:
					log.error("Failed to load data of message: %s" % e)
					continue
				if data:
					try:
						data = json.loads(data)
					except Exception as e:
						log.error("Failed to load json data '%s': %s" % (data, e))
						self._send_data(conn, self._create_error_responce(-32700, "Parse error", str(e)))
						continue
					if type(data) not in (tuple, list, dict):
						log.error("Wrong format of call")
						self._send_data(conn, self._create_error_responce(-32700, "Parse error", str(e)))
						continue
					if type(data) in (tuple, list):
						if len(data) == 0:
							self._send_data(conn, self._create_error_responce(-32600, "Invalid Request", str(e)))
							continue
						res = []
						for req in data:
							r = self._process_request(req)
							if r:
								res.append(r)
						if res:
							self._send_data(conn, res)
					else:
						res = self._process_request(data)
						if r:
							self._send_data(conn, res)
			else:
				return
		



import os
from . import base
from .decorators import *
import tuned.logs
from tuned.plugins import exceptions
from tuned.utils.commands import commands

log = tuned.logs.get()

class SelinuxPlugin(base.Plugin):
	"""
	`selinux`::
	
	Plug-in for tuning SELinux options.
	+
	SELinux decisions, such as allowing or denying access, are
	cached. This cache is known as the Access Vector Cache (AVC). When
	using these cached decisions, SELinux policy rules need to be checked
	less, which increases performance. The [option]`avc_cache_threshold`
	option allows adjusting the maximum number of AVC entries.
	+
	NOTE: Prior to changing the default value, evaluate the system
	performance with care. Increasing the value could potentially
	decrease the performance by making AVC slow.
	+
	.Increase the AVC cache threshold for hosts with containers.
	====
	----
	[selinux]
	avc_cache_threshold=8192
	----
	====
	"""

	@classmethod
	def _get_selinux_path(self):
		path = "/sys/fs/selinux"
		if not os.path.exists(path):
			path = "/selinux"
			if not os.path.exists(path):
				path = None
		return path

	def __init__(self, *args, **kwargs):
		self._cmd = commands()
		self._selinux_path = self._get_selinux_path()
		if self._selinux_path is None:
			raise exceptions.NotSupportedPluginException("SELinux is not enabled on your system or incompatible version is used.")
		self._cache_threshold_path = os.path.join(self._selinux_path, "avc", "cache_threshold")
		super(SelinuxPlugin, self).__init__(*args, **kwargs)

	@classmethod
	def _get_config_options(self):
		return {
			"avc_cache_threshold" : None,
		}

	def _instance_init(self, instance):
		instance._has_static_tuning = True
		instance._has_dynamic_tuning = False

	def _instance_cleanup(self, instance):
		pass

	@command_set("avc_cache_threshold")
	def _set_avc_cache_threshold(self, value, sim):
		if value is None:
			return None
		threshold = int(value)
		if threshold >= 0:
			if not sim:
				self._cmd.write_to_file(self._cache_threshold_path, threshold)
			return threshold
		else:
			return None

	@command_get("avc_cache_threshold")
	def _get_avc_cache_threshold(self):
		value = self._cmd.read_file(self._cache_threshold_path)
		if len(value) > 0:
			return int(value)
		return None



import os
import tuned.logs
from . import base
from tuned.utils.commands import commands

class kb2s(base.Function):
	"""
	Conversion function: kbytes to sectors
	"""
	def __init__(self):
		# 1 argument
		super(kb2s, self).__init__("kb2s", 1, 1)

	def execute(self, args):
		if not super(kb2s, self).execute(args):
			return None
		try:
			return str(int(args[0]) * 2)
		except ValueError:
			return None



import unittest
import tempfile
import shutil
import re
import os

from tuned.utils.commands import commands
import tuned.consts as consts
from tuned.exceptions import TunedException
import tuned.utils.commands

class CommandsTestCase(unittest.TestCase):
	def setUp(self):
		self._commands = commands()
		self._test_dir = tempfile.mkdtemp()
		self._test_file = tempfile.NamedTemporaryFile(mode='r',dir = self._test_dir)

	def test_get_bool(self):
		positive_values = ['y','yes','t','true']
		negative_values = ['n','no','f','false']

		for val in positive_values:
			self.assertEqual(self._commands.get_bool(val),"1")

		for val in negative_values:
			self.assertEqual(self._commands.get_bool(val),"0")

		self.assertEqual(self._commands.get_bool('bad_value'),'bad_value')

	def test_remove_ws(self):
		self.assertEqual(self._commands.remove_ws(' a  bc '),'a bc')

	def test_unquote(self):
		self.assertEqual(self._commands.unquote('"whatever"'),'whatever')

	def test_escape(self):
		self.assertEqual(self._commands.escape('\\'),'\\\\')

	def test_unescape(self):
		self.assertEqual(self._commands.unescape('\\'),'')

	def test_align_str(self):
		self.assertEqual(self._commands.align_str('abc',5,'def'),'abc  def')

	def test_dict2list(self):
		dictionary = {'key1':1,'key2':2,'key3':3}
		self.assertEqual(self._commands.dict2list(dictionary)\
			,['key1',1,'key2',2,'key3',3])

	def test_re_lookup_compile(self):
		pattern = re.compile(r'([1-9])')
		dictionary = {'[1-9]':''}
		self.assertEqual(self._commands.re_lookup_compile(dictionary).pattern\
			,pattern.pattern)
		self.assertIsNone(self._commands.re_lookup_compile(None))

	def test_multiple_re_replace(self):
		text = 'abcd1234'
		dictionary = {'abc':'gfh'}
		pattern = self._commands.re_lookup_compile(dictionary)
		self.assertEqual(self._commands.multiple_re_replace(dictionary,text)\
			,'gfhd1234')
		self.assertEqual(self._commands.multiple_re_replace(\
			dictionary,text,pattern),'gfhd1234')

	def test_re_lookup(self):
		dictionary = {'abc':'abc','mno':'mno'}
		text1 = 'abc def'
		text2 = 'jkl mno'
		text12 = 'abc mno'
		text3 = 'whatever'
		self.assertEqual(self._commands.re_lookup(dictionary,text1),'abc')
		self.assertEqual(self._commands.re_lookup(dictionary,text2),'mno')
		self.assertEqual(self._commands.re_lookup(dictionary,text12),'abc')
		self.assertIsNone(self._commands.re_lookup(dictionary,text3),None)

	def test_write_to_file(self):
		self.assertTrue(self._commands.write_to_file(self._test_file.name,\
			'hello world'))
		with open(self._test_file.name,'r') as f:
			self.assertEqual(f.read(),'hello world')

		self.assertTrue(self._commands.write_to_file(self._test_file.name,\
			'world hello'))
		with open(self._test_file.name,'r') as f:
			self.assertEqual(f.read(),'world hello')

		local_test_file = self._test_dir + '/dir' +'/self._test_file'
		self.assertTrue(self._commands.write_to_file(local_test_file,\
			'hello world',True))
		with open(local_test_file,'r') as f:
			self.assertEqual(f.read(),'hello world')

		shutil.rmtree(os.path.dirname(local_test_file))

		self.assertFalse(self._commands.write_to_file(local_test_file,\
			'hello world'))

	def test_read_file(self):
		with open(self._test_file.name,'w') as f:
			f.write('hello world')
		self.assertEqual(self._commands.read_file(self._test_file.name),\
			'hello world')
		self.assertEqual(self._commands.read_file('/bad_name','error'),\
			'error')

	def test_rmtree(self):
		test_tree = self._test_dir + '/one/two'
		os.makedirs(test_tree)
		test_tree = self._test_dir + '/one'
		self.assertTrue(self._commands.rmtree(test_tree))
		self.assertFalse(os.path.isdir(test_tree))
		self.assertTrue(self._commands.rmtree(test_tree))

	def test_unlink(self):
		local_test_file = self._test_dir + 'file_to_delete'
		open(local_test_file,'w').close()
		self.assertTrue(os.path.exists(local_test_file))
		self.assertTrue(self._commands.unlink(local_test_file))
		self.assertFalse(os.path.exists(local_test_file))
		self.assertTrue(self._commands.unlink(local_test_file))

	def test_rename(self):
		rename_test_file = self._test_dir + '/bad_name'
		open(rename_test_file,'w').close()
		self.assertTrue(self._commands.rename(rename_test_file,\
			self._test_dir + '/right_name'))
		self.assertTrue(os.path.exists(self._test_dir + '/right_name'))
		os.remove(self._test_dir + '/right_name')

	def test_copy(self):
		copy_test_file = self._test_dir + '/origo'
		with open(copy_test_file,'w') as f:
			f.write('hello world')
		self.assertTrue(self._commands.copy(copy_test_file,\
			self._test_dir + '/copy'))
		self.assertTrue(os.path.exists(self._test_dir + '/copy'))
		self.assertTrue(os.path.exists(self._test_dir + '/origo'))
		with open(self._test_dir + '/copy','r') as f:
			self.assertEqual(f.read(),'hello world')
		os.remove(self._test_dir + '/origo')
		os.remove(self._test_dir + '/copy')

	def test_replace_in_file(self):
		with open(self._test_file.name,'w') as f:
			f.write('hello world')

		self.assertTrue(self._commands.replace_in_file(self._test_file.name,\
			'hello','bye'))
		with open(self._test_file.name,'r') as f:
			self.assertEqual(f.read(),'bye world')

	def test_multiple_replace_in_file(self):
		dictionary = {'abc':'123','ghi':'456'}

		with open(self._test_file.name,'w') as f:
			f.write('abc def ghi')

		self.assertTrue(self._commands.multiple_replace_in_file(\
			self._test_file.name,dictionary))
		with open(self._test_file.name,'r') as f:
			self.assertEqual(f.read(),'123 def 456')

	def test_add_modify_option_in_file(self):
		with open(self._test_file.name,'w') as f:
			f.write('option1="123"\noption2="456"\n')

		dictionary = {'option3':789,'option1':321}
		self.assertTrue(self._commands.add_modify_option_in_file(\
			self._test_file.name,dictionary))
		with open(self._test_file.name,'r') as f:
			self.assertEqual(f.read(),\
				'option1="321"\noption2="456"\noption3="789"\n')

	def test_get_active_option(self):
		self.assertEqual(self._commands.get_active_option('opt1 [opt2] opt3'),\
			'opt2')
		self.assertEqual(self._commands.get_active_option('opt1 opt2 opt3'),\
			'opt1')
		self.assertEqual(self._commands.get_active_option(\
			'opt1 opt2 opt3',False),'opt1 opt2 opt3')

	def test_hex2cpulist(self):
		self.assertEqual(self._commands.hex2cpulist('0xf'),[0,1,2,3])
		self.assertEqual(self._commands.hex2cpulist('0x1,0000,0001'),[0,32])

	def test_cpulist_unpack(self):
		cpus = '4-8,^6,0xf00,,!10-11'
		self.assertEqual(self._commands.cpulist_unpack(cpus),[4,5,7,8,9])
		cpus = '\'\'"4-8,^6\'"'
		self.assertEqual(self._commands.cpulist_unpack(cpus, None),[])
		cpus = '\'\'"4-8,^6\'"'
		self.assertEqual(self._commands.cpulist_unpack(cpus),[4,5,7,8])
		cpus = '"4-8\',^6\'"'
		self.assertEqual(self._commands.cpulist_unpack(cpus),[])
		cpus = '1,2,3-x'
		self.assertEqual(self._commands.cpulist_unpack(cpus),[])
		cpus = '1,2,!3-x'
		self.assertEqual(self._commands.cpulist_unpack(cpus),[])
		cpus = {"1": "1"}
		self.assertEqual(self._commands.cpulist_unpack(cpus),[])

	def test_cpulist_pack(self):
		self.assertEqual(self._commands.cpulist_pack([0,1,3,4,5,6,8,9,32]),\
			['0-1','3-6','8-9','32'])

	def test_cpulist2hex(self):
		self.assertEqual(self._commands.cpulist2hex('1-3,5,32'),\
			'00000001,0000002e')

	def test_cpulist2bitmask(self):
		self.assertEqual(self._commands.cpulist2bitmask([1,2,3]),0b1110)
		self.assertEqual(self._commands.cpulist2bitmask([2,4,6]),0b1010100)

	def test_get_size(self):
		self.assertEqual(self._commands.get_size('100KB'),102400)
		self.assertEqual(self._commands.get_size('100Kb'),102400)
		self.assertEqual(self._commands.get_size('100kb'),102400)
		self.assertEqual(self._commands.get_size('1MB'),1024 * 1024)
		self.assertEqual(self._commands.get_size('1GB'),1024 * 1024 * 1024)

	def test_get_active_profile(self):
		consts.ACTIVE_PROFILE_FILE = self._test_dir + '/active_profile'
		consts.PROFILE_MODE_FILE = self._test_dir + '/profile_mode'
		with open(consts.ACTIVE_PROFILE_FILE,'w') as f:
			f.write('test_profile')
		with open(consts.PROFILE_MODE_FILE,'w') as f:
			f.write('auto')
		(profile,mode) = self._commands.get_active_profile()
		self.assertEqual(profile,'test_profile')
		self.assertEqual(mode,False)
		os.remove(consts.ACTIVE_PROFILE_FILE)
		os.remove(consts.PROFILE_MODE_FILE)
		(profile,mode) = self._commands.get_active_profile()
		self.assertEqual(profile,None)
		self.assertEqual(mode,None)

	def test_save_active_profile(self):
		consts.ACTIVE_PROFILE_FILE = self._test_dir + '/active_profile'
		consts.PROFILE_MODE_FILE = self._test_dir + '/profile_mode'
		self._commands.save_active_profile('test_profile',False)
		with open(consts.ACTIVE_PROFILE_FILE) as f:
			self.assertEqual(f.read(),'test_profile\n')
		with open(consts.PROFILE_MODE_FILE) as f:
			self.assertEqual(f.read(),'auto\n')
		os.remove(consts.ACTIVE_PROFILE_FILE)
		os.remove(consts.PROFILE_MODE_FILE)

	def tearDown(self):
		self._test_file.close()
		shutil.rmtree(self._test_dir)



#!/usr/bin/python3 -Es
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008-2013 Red Hat, Inc.
# Authors: Jan Kaluža <jkaluza@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

from __future__ import print_function
# exception handler for python 2/3 compatibility
try:
	from builtins import chr
except ImportError:
	pass
import os
import sys
import tempfile
import shutil
import argparse
import codecs
from subprocess import *
# exception handler for python 2/3 compatibility
try:
	from html.parser import HTMLParser
	from html.entities import name2codepoint
except ImportError:
	from HTMLParser import HTMLParser
	from htmlentitydefs import name2codepoint


SCRIPT_SH = """#!/bin/sh

. /usr/lib/tuned/functions

start() {
%s
	return 0
}

stop() {
%s
	return 0
}

process $@
"""

TUNED_CONF_PROLOG = "# Automatically generated by powertop2tuned tool\n\n"
TUNED_CONF_INCLUDE = """[main]
%s\n
"""
TUNED_CONF_EPILOG="""\n[powertop_script]
type=script
replace=1
script=${i:PROFILE_DIR}/script.sh
"""


class PowertopHTMLParser(HTMLParser):
	def __init__(self, enable_tunings):
		HTMLParser.__init__(self)

		self.inProperTable = False
		self.inScript = False
		self.intd = False
		self.lastStartTag = ""
		self.tdCounter = 0
		self.lastDesc = ""
		self.data = ""
		self.currentScript = ""
		if enable_tunings:
			self.prefix = ""
		else:
			self.prefix = "#"

		self.plugins = {}

	def getParsedData(self):
		return self.data

	def getPlugins(self):
		return self.plugins

	def handle_starttag(self, tag, attrs):
		self.lastStartTag = tag
		if self.lastStartTag == "div" and dict(attrs).get("id")  in ["tuning", "wakeup"]:
			self.inProperTable = True
		if self.inProperTable and tag == "td":
			self.tdCounter += 1
			self.intd = True

	def parse_command(self, command):
		prefix = ""
		command = command.strip()
		if command[0] == '#':
			prefix = "#"
			command = command[1:]

		if command.startswith("echo") and command.find("/proc/sys") != -1:
			splitted = command.split("'")
			value = splitted[1]
			path = splitted[3]
			path = path.replace("/proc/sys/", "").replace("/", ".")
			self.plugins.setdefault("sysctl", "[sysctl]\n")
			self.plugins["sysctl"] += "#%s\n%s%s=%s\n\n" % (self.lastDesc, prefix, path, value)
		# TODO: plugins/plugin_sysfs.py doesn't support this so far, it has to be implemented to 
		# let it work properly.
		elif command.startswith("echo") and (command.find("'/sys/") != -1 or command.find("\"/sys/") != -1):
			splitted = command.split("'")
			value = splitted[1]
			path = splitted[3]
			if path in ("/sys/module/snd_hda_intel/parameters/power_save", "/sys/module/snd_ac97_codec/parameters/power_save"):
				self.plugins.setdefault("audio", "[audio]\n")
				self.plugins["audio"] += "#%s\n%stimeout=1\n" % (self.lastDesc, prefix)
			else:
				self.plugins.setdefault("sysfs", "[sysfs]\n")
				self.plugins["sysfs"] += "#%s\n%s%s=%s\n\n" % (self.lastDesc, prefix, path, value)
		elif command.startswith("ethtool -s ") and command.endswith("wol d;"):
			self.plugins.setdefault("net", "[net]\n")
			self.plugins["net"] += "#%s\n%swake_on_lan=0\n" % (self.lastDesc, prefix)
		else:
			return False
		return True

	def handle_endtag(self, tag):
		if self.inProperTable and tag == "table":
			self.inProperTable = False
			self.intd = False
		if tag == "tr":
			self.tdCounter = 0
			self.intd = False
		if tag == "td":
			self.intd = False
		if self.inScript:
			#print self.currentScript
			self.inScript = False
			# Command is not handled, so just store it in the script
			if not self.parse_command(self.currentScript.split("\n")[-1]):
				self.data += self.currentScript + "\n\n"

	def handle_entityref(self, name):
		if self.inScript:
			self.currentScript += chr(name2codepoint[name])

	def handle_data(self, data):
		prefix = self.prefix
		if self.inProperTable and self.intd and self.tdCounter == 1:
			self.lastDesc = data
			if self.lastDesc.lower().find("autosuspend") != -1 and (self.lastDesc.lower().find("keyboard") != -1 or self.lastDesc.lower().find("mouse") != -1):
					self.lastDesc += "\n# WARNING: For some devices, uncommenting this command can disable the device."
					prefix = "#"
		if self.intd and ((self.inProperTable and self.tdCounter == 2) or self.inScript):
			self.tdCounter = 0
			if not self.inScript:
				self.currentScript += "\t# " + self.lastDesc + "\n"
				self.currentScript += "\t" + prefix + data.strip()
				self.inScript = True
			else:
				self.currentScript += data.strip()

class PowertopProfile:
	BAD_PRIVS = 100
	PARSING_ERROR = 101
	BAD_SCRIPTSH = 102

	def __init__(self, output, profile_name, name = ""):
		self.profile_name = profile_name
		self.name = name
		self.output = output

	def currentActiveProfile(self):
		proc = Popen(["tuned-adm", "active"], stdout=PIPE, \
				universal_newlines = True)
		output = proc.communicate()[0]
		if output and output.find("Current active profile: ") == 0:
			return output[len("Current active profile: "):output.find("\n")]
		return None

	def checkPrivs(self):
		myuid = os.geteuid()
		if myuid != 0:
			print('Run this program as root', file=sys.stderr)
			return False
		return True

	def generateHTML(self):
		print("Running PowerTOP, please wait...")
		environment = os.environ.copy()
		environment["LC_ALL"] = "C"
		try:
			proc = Popen(["/usr/sbin/powertop", \
					"--html=/tmp/powertop", "--time=1"], \
					stdout=PIPE, stderr=PIPE, \
					env=environment, \
					universal_newlines = True)
			output = proc.communicate()[1]
		except (OSError, IOError):
			print('Unable to execute PowerTOP, is PowerTOP installed?', file=sys.stderr)
			return -2

		if proc.returncode != 0:
			print('PowerTOP returned error code: %d' % proc.returncode, file=sys.stderr)
			return -2

		prefix = "PowerTOP outputting using base filename "
		if output.find(prefix) == -1:
			# workaround for PowerTOP older than 2.13
			prefix = "PowerTOP outputing using base filename "
			if output.find(prefix) == -1:
				return -1

		name = output[output.find(prefix)+len(prefix):-1]
		#print "Parsed filename=", [name]
		return name

	def parseHTML(self, enable_tunings):
		f = None
		data = None
		parser = PowertopHTMLParser(enable_tunings)
		try:
			f = codecs.open(self.name, "r", "utf-8")
			data = f.read()
		except (OSError, IOError, UnicodeDecodeError):
			data = None

		if f is not None:
			f.close()

		if data is None:
			return "", ""

		parser.feed(data)
		return parser.getParsedData(), parser.getPlugins()

	def generateShellScript(self, data):
		print("Generating shell script", os.path.join(self.output, "script.sh"))
		f = None
		try:
			f = codecs.open(os.path.join(self.output, "script.sh"), "w", "utf-8")
			f.write(SCRIPT_SH % (data, ""))
			os.fchmod(f.fileno(), 0o755)
			f.close()
		except (OSError, IOError) as e:
			print("Error writing shell script: %s" % e, file=sys.stderr)
			if f is not None:
				f.close()
			return False
		return True

	def generateTunedConf(self, profile, plugins):
		print("Generating TuneD config file", os.path.join(self.output, "tuned.conf"))
		f = codecs.open(os.path.join(self.output, "tuned.conf"), "w", "utf-8")
		f.write(TUNED_CONF_PROLOG)
		if profile is not None:
			if self.profile_name == profile:
				print('New profile has same name as active profile, not including active profile (avoiding circular deps).', file=sys.stderr)
			else:
				f.write(TUNED_CONF_INCLUDE % ("include=" + profile))

		for plugin in list(plugins.values()):
			f.write(plugin + "\n")

		f.write(TUNED_CONF_EPILOG)
		f.close()

	def generate(self, new_profile, merge_profile, enable_tunings):
		generated_html = False
		if len(self.name) == 0:
			generated_html = True
			if not self.checkPrivs():
				return self.BAD_PRIVS

			name = self.generateHTML()
			if isinstance(name, int):
				return name
			self.name = name

		data, plugins = self.parseHTML(enable_tunings)

		if generated_html:
			os.unlink(self.name)

		if len(data) == 0 and len(plugins) == 0:
			print('Your Powertop version is incompatible (maybe too old) or the generated HTML output is malformed', file=sys.stderr)
			return self.PARSING_ERROR

		if new_profile is False:
			if merge_profile is None:
				profile = self.currentActiveProfile()
			else:
				profile = merge_profile
		else:
			profile = None

		if not os.path.exists(self.output):
			os.makedirs(self.output)

		if not self.generateShellScript(data):
			return self.BAD_SCRIPTSH

		self.generateTunedConf(profile, plugins)

		return 0

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description='Creates TuneD profile from Powertop HTML output.')
	parser.add_argument('profile', metavar='profile_name', type=str, nargs='?', help='Name for the profile to be written.')
	parser.add_argument('-i', '--input', metavar='input_html', type=str, help='Path to Powertop HTML report. If not given, it is generated automatically.')
	parser.add_argument('-o', '--output', metavar='output_directory', type=str, help='Directory where the profile will be written, default is /etc/tuned/profile_name directory.')
	parser.add_argument('-n', '--new-profile', action='store_true', help='Creates new profile, otherwise it merges (include) your current profile.')
	parser.add_argument('-m', '--merge-profile', action = 'store', help = 'Merges (includes) the specified profile (can be suppressed by -n option).')
	parser.add_argument('-f', '--force', action='store_true', help='Overwrites the output directory if it already exists.')
	parser.add_argument('-e', '--enable', action='store_true', help='Enable all tunings (not recommended). Even with this enabled tunings known to be harmful (like USB_AUTOSUSPEND) won''t be enabled.')
	args = parser.parse_args()
	args = vars(args)

	if not args['profile'] and not args['output']:
		print('You have to specify the profile_name or output directory using the --output argument.', file=sys.stderr)
		parser.print_help()
		sys.exit(-1)

	if not args['output']:
		args['output'] = "/etc/tuned"

	if args['profile']:
		args['output'] = os.path.join(args['output'], args['profile'])

	if not args['input']:
		args['input'] = ''

	if os.path.exists(args['output']) and not args['force']:
		print('Output directory already exists, use --force to overwrite it.', file=sys.stderr)
		sys.exit(-1)

	p = PowertopProfile(args['output'], args['profile'], args['input'])
	sys.exit(p.generate(args['new_profile'], args['merge_profile'], args['enable']))



__all__ = ["ethcard"]

import tuned.logs
from subprocess import *
import re

log = tuned.logs.get()

class Nettool:

	_advertise_values = { # [ half, full ]
		10 : [ 0x001, 0x002 ],
		100 : [ 0x004, 0x008 ],
		1000 : [ 0x010, 0x020 ],
		2500 : [ 0, 0x8000 ], 
		10000 : [ 0, 0x1000 ],
		"auto" : 0x03F
	}

	_disabled = False

	def __init__(self, interface):
		self._interface = interface;
		self.update()

		log.debug("%s: speed %s, full duplex %s, autoneg %s, link %s" % (interface, self.speed, self.full_duplex, self.autoneg, self.link)) 
		log.debug("%s: supports: autoneg %s, modes %s" % (interface, self.supported_autoneg, self.supported_modes))
		log.debug("%s: advertises: autoneg %s, modes %s" % (interface, self.advertised_autoneg, self.advertised_modes))

#	def __del__(self):
#		if self.supported_autoneg:
#			self._set_advertise(self._advertise_values["auto"])

	def _clean_status(self):
		self.speed = 0
		self.full_duplex = False
		self.autoneg = False
		self.link = False

		self.supported_modes = []
		self.supported_autoneg = False

		self.advertised_modes = []
		self.advertised_autoneg = False

	def _calculate_mode(self, modes):
		mode = 0;
		for m in modes:
			mode += self._advertise_values[m[0]][ 1 if m[1] else 0 ]

		return mode

	def _set_autonegotiation(self, enable):
		if self.autoneg == enable:
			return True

		if not self.supported_autoneg:
			return False

		return 0 == call(["ethtool", "-s", self._interface, "autoneg", "on" if enable else "off"], close_fds=True)

	def _set_advertise(self, value):
		if not self._set_autonegotiation(True):
			return False

		return 0 == call(["ethtool", "-s", self._interface, "advertise", "0x%03x" % value], close_fds=True)

	def get_max_speed(self):
		max = 0
		for mode in self.supported_modes:
			if mode[0] > max: max = mode[0]

		if max > 0:
			return max
		else:
			return 1000

	def set_max_speed(self):
		if self._disabled or not self.supported_autoneg:
			return False

		#if self._set_advertise(self._calculateMode(self.supported_modes)):
		if self._set_advertise(self._advertise_values["auto"]):
			self.update()
			return True
		else:
			return False

	def set_speed(self, speed):
		if self._disabled or not self.supported_autoneg:
			return False

		mode = 0
		for am in self._advertise_values:
			if am == "auto": continue
			if am <= speed:
				mode += self._advertise_values[am][0];
				mode += self._advertise_values[am][1];

		effective_mode = mode & self._calculate_mode(self.supported_modes)

		log.debug("%s: set_speed(%d) - effective_mode 0x%03x" % (self._interface, speed, effective_mode))

		if self._set_advertise(effective_mode):
			self.update()
			return True
		else:
			return False

	def update(self):
		if self._disabled:
			return

		# run ethtool and preprocess output

		p_ethtool = Popen(["ethtool", self._interface], \
				stdout=PIPE, stderr=PIPE, close_fds=True, \
				universal_newlines = True)
		p_filter = Popen(["sed", "s/^\s*//;s/:\s*/:\\n/g"], \
				stdin=p_ethtool.stdout, stdout=PIPE, \
				universal_newlines = True, \
				close_fds=True)

		output = p_filter.communicate()[0]
		errors = p_ethtool.communicate()[1]

		if errors != "":
			log.warning("%s: some errors were reported by 'ethtool'" % self._interface)
			log.debug("%s: %s" % (self._interface, errors.replace("\n", r"\n")))
			self._clean_status()
			self._disabled = True
			return

		# parses output - kind of FSM

		self._clean_status()

		re_speed = re.compile(r"(\d+)")
		re_mode = re.compile(r"(\d+)baseT/(Half|Full)")

		state = "wait"

		for line in output.split("\n"):

			if line.endswith(":"):
				section = line[:-1]
				if section == "Speed": state = "speed"
				elif section == "Duplex": state = "duplex"
				elif section == "Auto-negotiation": state = "autoneg"
				elif section == "Link detected": state = "link"
				elif section == "Supported link modes": state = "supported_modes"
				elif section == "Supports auto-negotiation": state = "supported_autoneg"
				elif section == "Advertised link modes": state = "advertised_modes"
				elif section == "Advertised auto-negotiation": state = "advertised_autoneg"
				else: state = "wait"
				del section

			elif state == "speed":
				# Try to determine speed. If it fails, assume 1gbit ethernet
				try:
					self.speed = re_speed.match(line).group(1)
				except:
					self.speed = 1000
				state = "wait"

			elif state == "duplex":
				self.full_duplex = line == "Full"
				state = "wait"

			elif state == "autoneg":
				self.autoneg = (line == "yes" or line == "on")
				state = "wait"

			elif state == "link":
				self.link = line == "yes"
				state = "wait"

			elif state == "supported_modes":
				# Try to determine supported modes. If it fails, assume 1gibt ethernet fullduplex works
				try:
					for m in line.split():
						(s, d) = re_mode.match(m).group(1,2)
						self.supported_modes.append( (int(s), d == "Full") )
					del m,s,d
				except:
					self.supported_modes.append((1000, True))
	

			elif state == "supported_autoneg":
				self.supported_autoneg = line == "Yes"
				state = "wait"

			elif state == "advertised_modes":
				# Try to determine advertised modes. If it fails, assume 1gibt ethernet fullduplex works
				try:
					if line != "Not reported":
						for m in line.split():
							(s, d) = re_mode.match(m).group(1,2)
							self.advertised_modes.append( (int(s), d == "Full") )
						del m,s,d
				except:
					self.advertised_modes.append((1000, True))

			elif state == "advertised_autoneg":
				self.advertised_autoneg = line == "Yes"
				state = "wait"

def ethcard(interface):
	if not interface in ethcard.list:
		ethcard.list[interface] = Nettool(interface)

	return ethcard.list[interface]

ethcard.list = {}




import logging

GLOBAL_CONFIG_FILE = "/etc/tuned/tuned-main.conf"
ACTIVE_PROFILE_FILE = "/etc/tuned/active_profile"
PROFILE_MODE_FILE = "/etc/tuned/profile_mode"
POST_LOADED_PROFILE_FILE = "/etc/tuned/post_loaded_profile"
PROFILE_FILE = "tuned.conf"
RECOMMEND_CONF_FILE = "/etc/tuned/recommend.conf"
DAEMONIZE_PARENT_TIMEOUT = 5
NAMESPACE = "com.redhat.tuned"
DBUS_BUS = NAMESPACE
DBUS_INTERFACE = "com.redhat.tuned.control"
DBUS_OBJECT = "/Tuned"
DEFAULT_PROFILE = "balanced"
DEFAULT_STORAGE_FILE = "/run/tuned/save.pickle"
LOAD_DIRECTORIES = ["/usr/lib/tuned", "/etc/tuned"]
PERSISTENT_STORAGE_DIR = "/var/lib/tuned"
PLUGIN_MAIN_UNIT_NAME = "main"
# Magic section header because ConfigParser does not support "headerless" config
MAGIC_HEADER_NAME = "this_is_some_magic_section_header_because_of_compatibility"
RECOMMEND_DIRECTORIES = ["/usr/lib/tuned/recommend.d", "/etc/tuned/recommend.d"]

TMP_FILE_SUFFIX = ".tmp"
# max. number of consecutive errors to give up
ERROR_THRESHOLD = 3

# bootloader plugin configuration
BOOT_DIR = "/boot"
GRUB2_CFG_FILES = ["/etc/grub2.cfg", "/etc/grub2-efi.cfg"]
GRUB2_CFG_DIR = "/etc/grub.d"
GRUB2_TUNED_TEMPLATE_NAME = "00_tuned"
GRUB2_TUNED_TEMPLATE_PATH = GRUB2_CFG_DIR + "/" + GRUB2_TUNED_TEMPLATE_NAME
GRUB2_TEMPLATE_HEADER_BEGIN = "### BEGIN /etc/grub.d/" + GRUB2_TUNED_TEMPLATE_NAME +  " ###"
GRUB2_TEMPLATE_HEADER_END = "### END /etc/grub.d/" + GRUB2_TUNED_TEMPLATE_NAME +  " ###"
GRUB2_TUNED_VAR = "tuned_params"
GRUB2_TUNED_INITRD_VAR = "tuned_initrd"
GRUB2_DEFAULT_ENV_FILE = "/etc/default/grub"
INITRD_IMAGE_DIR = "/boot"
BOOT_CMDLINE_TUNED_VAR = "TUNED_BOOT_CMDLINE"
BOOT_CMDLINE_INITRD_ADD_VAR = "TUNED_BOOT_INITRD_ADD"
BOOT_CMDLINE_KARGS_DELETED_VAR = "TUNED_BOOT_KARGS_DELETED"
BOOT_CMDLINE_FILE = "/etc/tuned/bootcmdline"
PETITBOOT_DETECT_DIR = "/sys/firmware/opal"
MACHINE_ID_FILE = "/etc/machine-id"
KERNEL_UPDATE_HOOK_FILE = "/usr/lib/kernel/install.d/92-tuned.install"
BLS_ENTRIES_PATH = "/boot/loader/entries"

# scheduler plugin configuration
# how many times retry to move tasks to parent cgroup on cgroup cleanup
CGROUP_CLEANUP_TASKS_RETRY = 10
PROCFS_MOUNT_POINT = "/proc"
DEF_CGROUP_MOUNT_POINT = "/sys/fs/cgroup/cpuset"
DEF_CGROUP_MODE = 0o770

# service plugin configuration
SERVICE_SYSTEMD_CFG_PATH = "/etc/systemd/system/%s.service.d"
DEF_SERVICE_CFG_DIR_MODE = 0o755

# modules plugin configuration
MODULES_FILE = "/etc/modprobe.d/tuned.conf"

# systemd plugin configuration
SYSTEMD_SYSTEM_CONF_FILE = "/etc/systemd/system.conf"
SYSTEMD_CPUAFFINITY_VAR = "CPUAffinity"

# irqbalance plugin configuration
IRQBALANCE_SYSCONFIG_FILE = "/etc/sysconfig/irqbalance"

# built-in functions configuration
SYSFS_CPUS_PATH = "/sys/devices/system/cpu"

# number of backups
LOG_FILE_COUNT = 2
LOG_FILE_MAXBYTES = 100*1000
LOG_FILE = "/var/log/tuned/tuned.log"
PID_FILE = "/run/tuned/tuned.pid"
SYSTEM_RELEASE_FILE = "/etc/system-release-cpe"
# prefix for functions plugins
FUNCTION_PREFIX = "function_"
# prefix for exported environment variables when calling scripts
ENV_PREFIX = "TUNED_"

# tuned-gui
PREFIX_PROFILE_FACTORY = "System"
PREFIX_PROFILE_USER = "User"

# After adding new option to tuned-main.conf add here its name with CFG_ prefix
# and eventually default value with CFG_DEF_ prefix (default is None)
# and function for check with CFG_FUNC_ prefix
# (see configobj for methods, default is get for string)
CFG_DAEMON = "daemon"
CFG_DYNAMIC_TUNING = "dynamic_tuning"
CFG_SLEEP_INTERVAL = "sleep_interval"
CFG_UPDATE_INTERVAL = "update_interval"
CFG_RECOMMEND_COMMAND = "recommend_command"
CFG_REAPPLY_SYSCTL = "reapply_sysctl"
CFG_DEFAULT_INSTANCE_PRIORITY = "default_instance_priority"
CFG_UDEV_BUFFER_SIZE = "udev_buffer_size"
CFG_LOG_FILE_COUNT = "log_file_count"
CFG_LOG_FILE_MAX_SIZE = "log_file_max_size"
CFG_UNAME_STRING = "uname_string"
CFG_CPUINFO_STRING = "cpuinfo_string"
CFG_ENABLE_DBUS = "enable_dbus"
CFG_ENABLE_UNIX_SOCKET = "enable_unix_socket"
CFG_UNIX_SOCKET_PATH = "unix_socket_path"
CFG_UNIX_SOCKET_SIGNAL_PATHS = "unix_socket_signal_paths"
CFG_UNIX_SOCKET_OWNERSHIP = "unix_socket_ownership"
CFG_UNIX_SOCKET_PERMISIONS = "unix_socket_permissions"
CFG_UNIX_SOCKET_CONNECTIONS_BACKLOG = "connections_backlog"
CFG_CPU_EPP_FLAG = "hwp_epp"

# no_daemon mode
CFG_DEF_DAEMON = True
CFG_FUNC_DAEMON = "getboolean"
# default configuration
CFG_DEF_DYNAMIC_TUNING = True
CFG_FUNC_DYNAMIC_TUNING = "getboolean"
# how long to sleep before checking for events (in seconds)
CFG_DEF_SLEEP_INTERVAL = 1
CFG_FUNC_SLEEP_INTERVAL = "getint"
# update interval for dynamic tuning (in seconds)
CFG_DEF_UPDATE_INTERVAL = 10
CFG_FUNC_UPDATE_INTERVAL = "getint"
# recommend command availability
CFG_DEF_RECOMMEND_COMMAND = True
CFG_FUNC_RECOMMEND_COMMAND = "getboolean"
# reapply system sysctl
CFG_DEF_REAPPLY_SYSCTL = True
CFG_FUNC_REAPPLY_SYSCTL = "getboolean"
# default instance priority
CFG_DEF_DEFAULT_INSTANCE_PRIORITY = 0
CFG_FUNC_DEFAULT_INSTANCE_PRIORITY = "getint"
# default pyudev.Monitor buffer size
CFG_DEF_UDEV_BUFFER_SIZE = 1024 * 1024
# default log file count
CFG_DEF_LOG_FILE_COUNT = 2
CFG_FUNC_LOG_FILE_COUNT = "getint"
# default log file max size
CFG_DEF_LOG_FILE_MAX_SIZE = 1024 * 1024
# default listening on dbus
CFG_DEF_ENABLE_DBUS = True
CFG_FUNC_ENABLE_DBUS = "getboolean"
# default listening on unix socket
# as it is not used commonly disabled by default
CFG_DEF_ENABLE_UNIX_SOCKET = False
CFG_FUNC_ENABLE_UNIX_SOCKET = "getboolean"
# default unix socket path
CFG_DEF_UNIX_SOCKET_PATH = "/run/tuned/tuned.sock"
CFG_DEF_UNIX_SOCKET_SIGNAL_PATHS = ""
# default unix socket ownership
# (uid and gid, python2 does not support names out of box, -1 leaves default)
CFG_DEF_UNIX_SOCKET_OWNERSHIP = "-1 -1"
# default unix socket permissions
CFG_DEF_UNIX_SOCKET_PERMISIONS = "0o600"
# default unix socket conections backlog
CFG_DEF_UNIX_SOCKET_CONNECTIONS_BACKLOG = "1024"
CFG_FUNC_UNIX_SOCKET_CONNECTIONS_BACKLOG = "getint"

PATH_CPU_DMA_LATENCY = "/dev/cpu_dma_latency"

# profile attributes which can be specified in the main section
PROFILE_ATTR_SUMMARY = "summary"
PROFILE_ATTR_DESCRIPTION = "description"

SIGNAL_PROFILE_CHANGED = "profile_changed"

STR_HINT_REBOOT = "you need to reboot for changes to take effect"

STR_VERIFY_PROFILE_DEVICE_VALUE_OK = "verify: passed: device %s: '%s' = '%s'"
STR_VERIFY_PROFILE_VALUE_OK = "verify: passed: '%s' = '%s'"
STR_VERIFY_PROFILE_OK = "verify: passed: '%s'"
STR_VERIFY_PROFILE_DEVICE_VALUE_MISSING = "verify: skipped, missing: device %s: '%s'"
STR_VERIFY_PROFILE_VALUE_MISSING = "verify: skipped, missing: '%s'"
STR_VERIFY_PROFILE_DEVICE_VALUE_FAIL = "verify: failed: device %s: '%s' = '%s', expected '%s'"
STR_VERIFY_PROFILE_VALUE_FAIL = "verify: failed: '%s' = '%s', expected '%s'"
STR_VERIFY_PROFILE_CMDLINE_FAIL = "verify: failed: cmdline arg '%s', expected '%s'"
STR_VERIFY_PROFILE_CMDLINE_FAIL_MISSING = "verify: failed: cmdline arg '%s' is missing, expected '%s'"
STR_VERIFY_PROFILE_FAIL = "verify: failed: '%s'"

# timout for tuned-adm operations in seconds
ADMIN_TIMEOUT = 600

# Strings for /etc/tuned/profile_mode specifying if the active profile
# was set automatically or manually
ACTIVE_PROFILE_AUTO = "auto"
ACTIVE_PROFILE_MANUAL = "manual"

LOG_LEVEL_CONSOLE = 60
LOG_LEVEL_CONSOLE_NAME = "CONSOLE"
CAPTURE_LOG_LEVEL = "console"
CAPTURE_LOG_LEVELS = {
		"debug": logging.DEBUG,
		"info": logging.INFO,
		"warn": logging.WARN,
		"error": logging.ERROR,
		"console": LOG_LEVEL_CONSOLE,
		"none": None,
		}



import tuned.consts as consts
from . import base
from .decorators import *
from subprocess import Popen,PIPE
import tuned.logs
from tuned.utils.commands import commands
import glob

log = tuned.logs.get()
cmd = commands()

class MountsPlugin(base.Plugin):
	"""
	`mounts`::
	
	Enables or disables barriers for mounts according to the value of the
	[option]`disable_barriers` option. The [option]`disable_barriers`
	option has an optional value `force` which disables barriers even
	on mountpoints with write back caches. Note that only extended file
	systems (ext) are supported by this plug-in.
	"""

	@classmethod
	def _generate_mountpoint_topology(cls):
		"""
		Gets the information about disks, partitions and mountpoints. Stores information about used filesystem and
		creates a list of all underlying devices (in case of LVM) for each mountpoint.
		"""
		mountpoint_topology = {}
		current_disk = None

		stdout, stderr = Popen(["lsblk", "-rno", \
				"TYPE,RM,KNAME,FSTYPE,MOUNTPOINT"], \
				stdout=PIPE, stderr=PIPE, close_fds=True, \
				universal_newlines = True).communicate()
		for columns in [line.split() for line in stdout.splitlines()]:
			if len(columns) < 3:
				continue
			device_type, device_removable, device_name = columns[:3]
			filesystem = columns[3] if len(columns) > 3 else None
			mountpoint = columns[4] if len(columns) > 4 else None

			if device_type == "disk":
				current_disk = device_name
				continue

			# skip removable, skip nonpartitions
			if device_removable == "1" or device_type not in ["part", "lvm"]:
				continue

			if mountpoint is None or mountpoint == "[SWAP]":
				continue

			mountpoint_topology.setdefault(mountpoint, {"disks": set(), "device_name": device_name, "filesystem": filesystem})
			mountpoint_topology[mountpoint]["disks"].add(current_disk)

		cls._mountpoint_topology = mountpoint_topology

	def _init_devices(self):
		self._generate_mountpoint_topology()
		self._devices_supported = True
		self._free_devices = set(self._mountpoint_topology.keys())
		self._assigned_devices = set()

	@classmethod
	def _get_config_options(self):
		return {
			"disable_barriers": None,
		}

	def _instance_init(self, instance):
		instance._has_dynamic_tuning = False
		instance._has_static_tuning = True

	def _instance_cleanup(self, instance):
		pass

	def _get_device_cache_type(self, device):
		"""
		Get device cache type. This will work only for devices on SCSI kernel subsystem.
		"""
		source_filenames = glob.glob("/sys/block/%s/device/scsi_disk/*/cache_type" % device)
		for source_filename in source_filenames:
			return cmd.read_file(source_filename).strip()
		return None

	def _mountpoint_has_writeback_cache(self, mountpoint):
		"""
		Checks if the device has 'write back' cache. If the cache type cannot be determined, asume some other cache.
		"""
		for device in self._mountpoint_topology[mountpoint]["disks"]:
			if self._get_device_cache_type(device) == "write back":
				return True
		return False

	def _mountpoint_has_barriers(self, mountpoint):
		"""
		Checks if a given mountpoint is mounted with barriers enabled or disabled.
		"""
		with open("/proc/mounts") as mounts_file:
			for line in mounts_file:
				# device mountpoint filesystem options dump check
				columns = line.split()
				if columns[0][0] != "/":
					continue
				if columns[1] == mountpoint:
					option_list = columns[3]
					break
			else:
				return None

		options = option_list.split(",")
		for option in options:
			(name, sep, value) = option.partition("=")
			# nobarrier barrier=0
			if name == "nobarrier" or (name == "barrier" and value == "0"):
				return False
			# barrier barrier=1
			elif name == "barrier":
				return True
		else:
			# default
			return True

	def _remount_partition(self, partition, options):
		"""
		Remounts partition.
		"""
		remount_command = ["/usr/bin/mount", partition, "-o", "remount,%s" % options]
		cmd.execute(remount_command)

	@command_custom("disable_barriers", per_device=True)
	def _disable_barriers(self, start, value, mountpoint, verify, ignore_missing):
		storage_key = self._storage_key(
				command_name = "disable_barriers",
				device_name = mountpoint)
		force = str(value).lower() == "force"
		value = force or self._option_bool(value)

		if start:
			if not value:
				return None

			reject_reason = None

			if not self._mountpoint_topology[mountpoint]["filesystem"].startswith("ext"):
				reject_reason = "filesystem not supported"
			elif not force and self._mountpoint_has_writeback_cache(mountpoint):
				reject_reason = "device uses write back cache"
			else:
				original_value = self._mountpoint_has_barriers(mountpoint)
				if original_value is None:
					reject_reason = "unknown current setting"
				elif original_value == False:
					if verify:
						log.info(consts.STR_VERIFY_PROFILE_OK % mountpoint)
						return True
					else:
						reject_reason = "barriers already disabled"
				elif verify:
					log.error(consts.STR_VERIFY_PROFILE_FAIL % mountpoint)
					return False

			if reject_reason is not None:
				log.info("not disabling barriers on '%s' (%s)" % (mountpoint, reject_reason))
				return None

			self._storage.set(storage_key, original_value)
			log.info("disabling barriers on '%s'" % mountpoint)
			self._remount_partition(mountpoint, "barrier=0")

		else:
			if verify:
				return None
			original_value = self._storage.get(storage_key)
			if original_value is None:
				return None

			log.info("enabling barriers on '%s'" % mountpoint)
			self._remount_partition(mountpoint, "barrier=1")
			self._storage.unset(storage_key)
		return None
